# coding=utf-8
"""
The List Member Events API endpoint

Documentation: https://mailchimp.com/developer/reference/lists/list-members/list-member-events/
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.helpers import check_subscriber_hash


class ListMemberEvents(BaseApi):
    """
    Use the Events endpoint to collect website or in-app actions and trigger targeted automations.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListMemberEvents, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None
        self.subscriber_hash = None

    def create(self, list_id, subscriber_hash, data):
        """
        Add an event for a list member.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "name": string*, (Must be 2-30 characters in length)
        }
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        if 'name' not in data:
            raise KeyError('The list member events must have a name')
        if len(data['name']) < 2 or len(data['name']) > 30:
            raise ValueError('The list member events name must be 2-30 in length')
        return self._mc_client._post(url=self._build_path(list_id, 'members', subscriber_hash, 'events'), data=data)

    def all(self, list_id, subscriber_hash, get_all=False, **queryparams):
        """
        Get events for a contact

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['count'] = integer
        queryparams['offset'] = integer
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        if get_all:
            return self._iterate(url=self._build_path(list_id, 'members', subscriber_hash, 'events'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(list_id, 'members', subscriber_hash, 'events'), **queryparams)



# coding=utf-8
"""
The List Growth History API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/growth-history/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Growth/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class ListGrowthHistory(BaseApi):
    """
    View a summary of the month-by-month growth activity for a specific list.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListGrowthHistory, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None
        self.month = None


    def all(self, list_id, get_all=False, **queryparams):
        """
        Get a month-by-month summary of a specific list’s growth activity.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.list_id = list_id
        self.month = None
        if get_all:
            return self._iterate(url=self._build_path(list_id, 'growth-history'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(list_id, 'growth-history'), **queryparams)


    def get(self, list_id, month, **queryparams):
        """
        Get a summary of a specific list’s growth activity for a specific month and year.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param month: A specific month of list growth history.
        :type month: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.list_id = list_id
        self.month = month
        return self._mc_client._get(url=self._build_path(list_id, 'growth-history', month), **queryparams)



# coding=utf-8
"""
Helper functions to perform simple tasks for multiple areas of the API
"""
import hashlib
import re

HTTP_METHOD_ACTION_MATCHING = {
    'get': 'GET',
    'create': 'POST',
    'update': 'PATCH',
    'create_or_update': 'PUT',
    'delete': 'DELETE'
}


def get_subscriber_hash(member_email):
    """
    The MD5 hash of the lowercase version of the list member's email.
    Used as subscriber_hash

    :param member_email: The member's email address
    :type member_email: :py:class:`str`
    :returns: The MD5 hash in hex
    :rtype: :py:class:`str`
    """
    check_email(member_email)
    member_email = member_email.lower().encode()
    m = hashlib.md5(member_email)
    return m.hexdigest()


def check_subscriber_hash(potential_hash):
    """
    Check the passed value to see if it matches a 32 character hex number that
    MD5 generates as output, or compute that value assuming that the input is
    an email address.

    :param potential_hash: A value to be passed to any of the endpoints that
    expect an MD5 of an email address
    :type potential_hash: :py:class:`str`
    :returns: A valid MD5 hash in hex
    :rtype: :py:class:`str`
    """
    if re.match(r"^[0-9a-f]{32}$", potential_hash):
        return potential_hash
    else:
        return get_subscriber_hash(potential_hash)


def check_email(email):
    """
    Function that verifies that the string passed is a valid email address.

    Regex for email validation based on MailChimp limits:
    http://kb.mailchimp.com/accounts/management/international-characters-in-mailchimp

    :param email: The potential email address
    :type email: :py:class:`str`
    :return: Nothing
    """
    if not re.match(r".+@.+\..+", email):
        raise ValueError('String passed is not a valid email address')
    return


def check_url(url):
    """
    Function that verifies that the string passed is a valid url.

    Original regex author Diego Perini (http://www.iport.it)
    regex ported to Python by adamrofer (https://github.com/adamrofer)
    Used under MIT license.

    :param url:
    :return: Nothing
    """
    URL_REGEX = re.compile(
    u"^"
    u"(?:(?:https?|ftp)://)"
    u"(?:\\S+(?::\\S*)?@)?"
    u"(?:"
    u"(?!(?:10|127)(?:\\.\\d{1,3}){3})"
    u"(?!(?:169\\.254|192\\.168)(?:\\.\\d{1,3}){2})"
    u"(?!172\\.(?:1[6-9]|2\\d|3[0-1])(?:\\.\\d{1,3}){2})"
    u"(?:[1-9]\\d?|1\\d\\d|2[01]\\d|22[0-3])"
    u"(?:\\.(?:1?\\d{1,2}|2[0-4]\\d|25[0-5])){2}"
    u"(?:\\.(?:[1-9]\\d?|1\\d\\d|2[0-4]\\d|25[0-4]))"
    u"|"
    u"(?:(?:[a-z\u00a1-\uffff0-9]-?)*[a-z\u00a1-\uffff0-9]+)"
    u"(?:\\.(?:[a-z\u00a1-\uffff0-9]-?)*[a-z\u00a1-\uffff0-9]+)*"
    u"(?:\\.(?:[a-z\u00a1-\uffff]{2,}))"
    u")"
    u"(?::\\d{2,5})?"
    u"(?:/\\S*)?"
    u"$"
    , re.UNICODE)
    if not re.match(URL_REGEX, url):
        raise ValueError('String passed is not a valid url')
    return


def merge_results(x, y):
    """
    Given two dicts, x and y, merge them into a new dict as a shallow copy.

    The result only differs from `x.update(y)` in the way that it handles list
    values when both x and y have list values for the same key. In which case
    the returned dictionary, z, has a value according to:
      z[key] = x[key] + z[key]

    :param x: The first dictionary
    :type x: :py:class:`dict`
    :param y: The second dictionary
    :type y: :py:class:`dict`
    :returns: The merged dictionary
    :rtype: :py:class:`dict`
    """
    z = x.copy()
    for key, value in y.items():
        if isinstance(value, list) and isinstance(z.get(key), list):
            z[key] += value
        else:
            z[key] = value
    return z



# coding=utf-8
"""
The Authorized Apps endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/authorized-apps/
Schema: http://api.mailchimp.com/schema/3.0/AuthorizedApps/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class AuthorizedApps(BaseApi):
    """
    Manage registered, connected apps for your MailChimp account with the
    Authorized Apps endpoints.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(AuthorizedApps, self).__init__(*args, **kwargs)
        self.endpoint = 'authorized-apps'
        self.app_id = None


    def create(self, data):
        """
        Retrieve OAuth2-based credentials to associate API calls with your
        application.

        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "client_id": string*,
            "client_secret": string*
        }
        """
        self.app_id = None
        if 'client_id' not in data:
            raise KeyError('The authorized app must have a client_id')
        if 'client_secret' not in data:
            raise KeyError('The authorized app must have a client_secret')
        return self._mc_client._post(url=self._build_path(), data=data)


    def all(self, get_all=False, **queryparams):
        """
        Get a list of an account’s registered, connected applications.

        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.app_id = None
        if get_all:
            return self._iterate(url=self._build_path(), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(), **queryparams)


    def get(self, app_id, **queryparams):
        """
        Get information about a specific authorized application

        :param app_id: The unique id for the connected authorized application
        :type app_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.app_id = app_id
        return self._mc_client._get(url=self._build_path(app_id), **queryparams)



# coding=utf-8
"""
The Search Members API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/search-members/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Members/Collection.json
Additional Data: http://kb.mailchimp.com/accounts/management/search-for-subscribers-and-campaigns-in-your-account
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class SearchMembers(BaseApi):
    """
    Manage campaign reports for your MailChimp account. All Reports endpoints
    are read-only. MailChimp’s campaign and Automation reports analyze clicks,
    opens, subscribers’ social activity, e-commerce data, and more.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(SearchMembers, self).__init__(*args, **kwargs)
        self.endpoint = 'search-members'
        self.list_id = None


    def get(self, **queryparams):
        """
        Search for list members. This search can be restricted to a specific
        list, or can be used to search across all lists in an account.

        :param queryparams: The query string parameters
        queryparams['fields'] = array
        queryparams['exclude_fields'] = array
        queryparams['query'] = string
        queryparams['list_id'] = string
        queryparams['offset'] = integer
        """
        if 'list_id' in queryparams:
            self.list_id = queryparams['list_id']
        else:
            self.list_id = None
        return self._mc_client._get(url=self._build_path(), **queryparams)



# coding=utf-8
"""
The List Interest Categories API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/interest-categories/
Schema: https://api.mailchimp.com/schema/3.0/Lists/InterestCategories/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.listinterestcategoryinterest import ListInterestCategoryInterest


class ListInterestCategories(BaseApi):
    """
    Manage interest categories for a specific list. Interest categories
    organize interests, which are used to group subscribers based on their
    preferences. These correspond to ‘group titles’ in the MailChimp
    application.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListInterestCategories, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None
        self.category_id = None
        self.interests = ListInterestCategoryInterest(self)


    def create(self, list_id, data):
        """
        Create a new interest category.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "title": string*,
            "type": string* (Must be one of 'checkboxes', 'dropdown', 'radio', or 'hidden')
        }
        """
        self.list_id = list_id
        if 'title' not in data:
            raise KeyError('The list interest category must have a title')
        if 'type' not in data:
            raise KeyError('The list interest category must have a type')
        if data['type'] not in ['checkboxes', 'dropdown', 'radio', 'hidden']:
            raise ValueError('The list interest category type must be one of "checkboxes", "dropdown", "radio", or '
                             '"hidden"')
        response = self._mc_client._post(url=self._build_path(list_id, 'interest-categories'), data=data)
        if response is not None:
            self.category_id = response['id']
        else:
            self.category_id = None
        return response


    def all(self, list_id, get_all=False, **queryparams):
        """
        Get information about a list’s interest categories.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        queryparams['type'] = string
        """
        self.list_id = list_id
        self.category_id = None
        if get_all:
            return self._iterate(url=self._build_path(list_id, 'interest-categories'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(list_id, 'interest-categories'), **queryparams)


    def get(self, list_id, category_id, **queryparams):
        """
        Get information about a specific interest category.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param category_id: The unique id for the interest category.
        :type category_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.list_id = list_id
        self.category_id = category_id
        return self._mc_client._get(url=self._build_path(list_id, 'interest-categories', category_id), **queryparams)


    def update(self, list_id, category_id, data):
        """
        Update a specific interest category.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param category_id: The unique id for the interest category.
        :type category_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "title": string*,
            "type": string* (Must be one of 'checkboxes', 'dropdown', 'radio', or 'hidden')
        }
        """
        self.list_id = list_id
        self.category_id = category_id
        if 'title' not in data:
            raise KeyError('The list interest category must have a title')
        if 'type' not in data:
            raise KeyError('The list interest category must have a type')
        if data['type'] not in ['checkboxes', 'dropdown', 'radio', 'hidden']:
            raise ValueError('The list interest category type must be one of "checkboxes", "dropdown", "radio", or '
                             '"hidden"')
        return self._mc_client._patch(url=self._build_path(list_id, 'interest-categories', category_id), data=data)


    def delete(self, list_id, category_id):
        """
        Delete a specific interest category.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param category_id: The unique id for the interest category.
        :type category_id: :py:class:`str`
        """
        self.list_id = list_id
        self.category_id = category_id
        return self._mc_client._delete(url=self._build_path(list_id, 'interest-categories', category_id))



# coding=utf-8
"""
The List Webhooks API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/webhooks/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Webhooks/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.helpers import check_url


class ListWebhooks(BaseApi):
    """
    Manage webhooks for a specific MailChimp list.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListWebhooks, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None
        self.webhook_id = None


    def create(self, list_id, data):
        """
        Create a new webhook for a specific list.

        The documentation does not include any required request body
        parameters but the url parameter is being listed here as a required
        parameter in documentation and error-checking based on the description
        of the method

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "url": string*
        }
        """
        self.list_id = list_id
        if 'url' not in data:
            raise KeyError('The list webhook must have a url')
        check_url(data['url'])
        response = self._mc_client._post(url=self._build_path(list_id, 'webhooks'), data=data)
        if response is not None:
            self.webhook_id = response['id']
        else:
            self.webhook_id = None
        return response


    def all(self, list_id):
        """
        Get information about all webhooks for a specific list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        """
        self.list_id = list_id
        self.webhook_id = None
        return self._mc_client._get(url=self._build_path(list_id, 'webhooks'))


    def get(self, list_id, webhook_id):
        """
        Get information about a specific webhook.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param webhook_id: The unique id for the webhook.
        :type webhook_id: :py:class:`str`
        """
        self.list_id = list_id
        self.webhook_id = webhook_id
        return self._mc_client._get(url=self._build_path(list_id, 'webhooks', webhook_id))


    def update(self, list_id, webhook_id, data):
        """
        Update the settings for an existing webhook.

        :param list_id: The unique id for the list
        :type list_id: :py:class:`str`
        :param webhook_id: The unique id for the webhook
        :type webhook_id: :py:class:`str`
        """
        self.list_id = list_id
        self.webhook_id = webhook_id
        return self._mc_client._patch(url=self._build_path(list_id, 'webhooks', webhook_id), data=data)


    def delete(self, list_id, webhook_id):
        """
        Delete a specific webhook in a list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param webhook_id: The unique id for the webhook.
        :type webhook_id: :py:class:`str`
        """
        self.list_id = list_id
        self.webhook_id = webhook_id
        return self._mc_client._delete(url=self._build_path(list_id, 'webhooks', webhook_id))



# coding=utf-8
"""
The E-commerce Stores Promo CodesAPI endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/ecommerce/stores/promo-rules
"""
from __future__ import unicode_literals
from mailchimp3.baseapi import BaseApi

class StorePromoCodes(BaseApi):
    """
        Promo Promo codes can be created for a given promo rule. All the promo codes under a promo rule share the
        generic information defined for that rule like the amount, type, expiration date etc.
        """
    def __init__(self, *args, **kwargs):
        """
        Initialize the Endpoint
        :param args:
        :param kwargs:
        """
        super(StorePromoCodes, self).__init__(*args, **kwargs)
        self.endpoint = 'ecommerce/stores'
        self.store_id = None

    def create(self, store_id, promo_rule_id, data):
        """
        Add a new promo code to a store.

        :param store_id: The store id
        :type store_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict'
        data = {
            "id": string*,
            "code": string*,
            "redemption_url": string*,
            "usage_count": string,
            "enabled": boolean,
            "created_at_foreign": string,
            "updated_at_foreign": string,
        }
        """
        self.store_id = store_id
        if 'id' not in data:
            raise KeyError('The promo code must have an id')
        if 'code' not in data:
            raise KeyError('This promo code must have a code')
        if 'redemption_url' not in data:
            raise KeyError('This promo code must have a redemption url')
        response = self._mc_client._post(url=self._build_path(store_id, 'promo-rules', promo_rule_id, 'promo-codes'), data=data)

        if response is not None:
            return response

    def all(self, store_id, promo_rule_id, get_all=False, **queryparams):
        """
        Get information about a store’s promo codes.

        :param store_id: The store's id
        :type store_id: `str`
        :param promo_rule_id: The store promo rule id
        :type store_id: `str`
        :param get_all:
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.store_id=store_id
        self.promo_rule_id=promo_rule_id
        if get_all:
            return self._iterate(url=self._build_path(store_id, 'promo-rules', promo_rule_id, 'promo-codes'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(store_id, 'promo-rule', promo_rule_id), **queryparams)

    def get(self, store_id, promo_rule_id, promo_code_id, **queryparams):
        """
                Get information about a specific promo code.

                :param store_id: The store's id
                :type store_id: `string`
                :param queryparams: The query string parameters
                queryparams['fields'] = []
                queryparams['exclude_fields'] = []
                queryparams['count'] = integer
                queryparams['offset'] = integer
                """
        self.store_id = store_id
        self.promo_rule_id = promo_rule_id
        self.promo_code_id = promo_code_id
        return self._mc_client._get(url=self._build_path(store_id, 'promo-rules', promo_rule_id, 'promo-codes', promo_code_id), **queryparams)


    def update(self, store_id, promo_rule_id, promo_code_id, data):
        """
        Update a promo code

        :param store_id: The store id
        :type :py:class:`str`
        :param promo_rule_id: The id for the promo rule of a store.
        :type :py:class:`str`
        :param promo_code_id: The id for the promo code of a store.
        :type :py:class:`str`
        :param data:
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "id": string,
            "title": string,
            "description": string,
            "starts_at": string,
            "ends_at": string,
            "amount": number,
            "type": string,
            "target": string,
            "enabled": boolean,
            "created_at_foreign": string,
            "updated_at_foreign": string,
        }
        """
        self.store_id = store_id
        self.promo_rule_id = promo_rule_id
        self.promo_code_id = promo_code_id
        return self._mc_client._patch(url=self._build_path(store_id, 'promo-rules', promo_rule_id, 'promo-codes', promo_code_id), data=data)

    def delete(self, store_id, promo_rule_id, promo_code_id):
        """
        Delete a promo code
        :param store_id: The store id
        :type :py:class:`str`
        :param promo_rule_id: The id for the promo rule of a store.
        :type :py:class:`str`
        :param promo_code_id: The id for the promo code of a store.
        :type :py:class:`str`
        """
        self.store_id=store_id
        self.promo_rule_id=promo_rule_id
        return self._mc_client._delete(url=self._build_path(store_id, 'promo-rules', promo_rule_id, 'promo-codes', promo_code_id))


# coding=utf-8
"""
The Batch Webhooks API Endpoint

Documentation: https://developer.mailchimp.com/documentation/mailchimp/reference/batch-webhooks/
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class BatchWebhooks(BaseApi):
    """
    Manage webhooks for batch operations.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(BatchWebhooks, self).__init__(*args, **kwargs)
        self.endpoint = 'batch-webhooks'
        self.batch_webhook_id = None


    def create(self, data):
        """
        Configure a webhook that will fire whenever any batch request
        completes processing.

        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "url": string*
        }
        """
        if 'url' not in data:
            raise KeyError('The batch webhook must have a valid url')
        response = self._mc_client._post(url=self._build_path(), data=data)
        if response is not None:
            self.batch_webhook_id = response['id']
        else:
            self.batch_webhook_id = None
        return response


    def all(self, get_all=False, **queryparams):
        """
        Get all webhooks that have been configured for batches.

        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.batch_webhook_id = None
        if get_all:
            return self._iterate(url=self._build_path(), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(), **queryparams)


    def get(self, batch_webhook_id, **queryparams):
        """
        Get information about a specific batch webhook.

        :param batch_webhook_id: The unique id for the batch webhook.
        :type batch_webhook_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.batch_webhook_id = batch_webhook_id
        return self._mc_client._get(url=self._build_path(batch_webhook_id), **queryparams)


    def update(self, batch_webhook_id, data):
        """
        Update a webhook that will fire whenever any batch request completes
        processing.

        :param batch_webhook_id: The unique id for the batch webhook.
        :type batch_webhook_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "url": string*
        }
        """
        self.batch_webhook_id = batch_webhook_id
        if 'url' not in data:
            raise KeyError('The batch webhook must have a valid url')
        return self._mc_client._patch(url=self._build_path(batch_webhook_id), data=data)


    def delete(self, batch_webhook_id):
        """
        Remove a batch webhook. Webhooks will no longer be sent to the given
        URL.

        :param batch_webhook_id: The unique id for the batch webhook.
        :type batch_webhook_id: :py:class:`str`
        """
        self.batch_webhook_id = batch_webhook_id
        return self._mc_client._delete(url=self._build_path(batch_webhook_id))



# coding=utf-8
"""
The List Members API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/members/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Members/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.listmemberactivity import ListMemberActivity
from mailchimp3.entities.listmemberevents import ListMemberEvents
from mailchimp3.entities.listmembergoals import ListMemberGoals
from mailchimp3.entities.listmembernotes import ListMemberNotes
from mailchimp3.helpers import check_email, check_subscriber_hash


class ListMembers(BaseApi):
    """
    Manage members of a specific MailChimp list, including currently
    subscribed, unsubscribed, and bounced members.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListMembers, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None
        self.subscriber_hash = None
        self.activity = ListMemberActivity(self)
        self.events = ListMemberEvents(self)
        self.goals = ListMemberGoals(self)
        self.notes = ListMemberNotes(self)


    def create(self, list_id, data):
        """
        Add a new member to the list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "status": string*, (Must be one of 'subscribed', 'unsubscribed', 'cleaned',
                'pending', or 'transactional')
            "email_address": string*
        }
        """
        self.list_id = list_id
        if 'status' not in data:
            raise KeyError('The list member must have a status')
        if data['status'] not in ['subscribed', 'unsubscribed', 'cleaned', 'pending', 'transactional']:
            raise ValueError('The list member status must be one of "subscribed", "unsubscribed", "cleaned", '
                             '"pending", or "transactional"')
        if 'email_address' not in data:
            raise KeyError('The list member must have an email_address')
        check_email(data['email_address'])
        response = self._mc_client._post(url=self._build_path(list_id, 'members'), data=data)
        if response is not None:
            self.subscriber_hash = response['id']
        else:
            self.subscriber_hash = None
        return response


    def all(self, list_id, get_all=False, **queryparams):
        """
        Get information about members in a specific MailChimp list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        queryparams['email_type'] = string
        queryparams['status'] = string
        queryparams['before_timestamp_opt'] = string
        queryparams['since_timestamp_opt'] = string
        queryparams['before_last_changed'] = string
        queryparams['since_last_changed'] = string
        queryparams['unique_email_id'] = string
        queryparams['vip_only'] = boolean
        queryparams['interest_category_id'] = string
        queryparams['interest_ids'] = string
        queryparams['interest_match'] = string
        """
        self.list_id = list_id
        self.subscriber_hash = None
        if get_all:
            return self._iterate(url=self._build_path(list_id, 'members'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(list_id, 'members'), **queryparams)


    def get(self, list_id, subscriber_hash, **queryparams):
        """
        Get information about a specific list member, including a currently
        subscribed, unsubscribed, or bounced member.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
            list member’s email address.
        :type subscriber_hash: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        return self._mc_client._get(url=self._build_path(list_id, 'members', subscriber_hash), **queryparams)


    def update(self, list_id, subscriber_hash, data):
        """
        Update information for a specific list member.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
            list member’s email address.
        :type subscriber_hash: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        return self._mc_client._patch(url=self._build_path(list_id, 'members', subscriber_hash), data=data)


    def create_or_update(self, list_id, subscriber_hash, data):
        """
        Add or update a list member.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
            list member’s email address.
        :type subscriber_hash: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "email_address": string*,
            "status_if_new": string* (Must be one of 'subscribed',
                'unsubscribed', 'cleaned', 'pending', or 'transactional')
        }
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        if 'email_address' not in data:
            raise KeyError('The list member must have an email_address')
        check_email(data['email_address'])
        if 'status_if_new' not in data:
            raise KeyError('The list member must have a status_if_new')
        if data['status_if_new'] not in ['subscribed', 'unsubscribed', 'cleaned', 'pending', 'transactional']:
            raise ValueError('The list member status_if_new must be one of "subscribed", "unsubscribed", "cleaned", '
                             '"pending", or "transactional"')
        return self._mc_client._put(url=self._build_path(list_id, 'members', subscriber_hash), data=data)


    def delete(self, list_id, subscriber_hash):
        """
        Delete a member from a list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        return self._mc_client._delete(url=self._build_path(list_id, 'members', subscriber_hash))

    def delete_permanent(self, list_id, subscriber_hash):
        """
        Delete permanently a member from a list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        return self._mc_client._post(url=self._build_path(list_id, 'members', subscriber_hash, 'actions', 'delete-permanent'))



# coding=utf-8
"""
Mailchimp v3 Api SDK

Documentation: http://developer.mailchimp.com/documentation/mailchimp/
"""
from __future__ import unicode_literals
import functools
import re


import requests
from requests.auth import HTTPBasicAuth
# Handle library reorganisation Python 2 > Python 3.
try:
    from urllib.parse import urljoin
    from urllib.parse import urlencode
except ImportError:
    from urlparse import urljoin
    from urllib import urlencode

import logging

_logger = logging.getLogger('mailchimp3.client')

def _enabled_or_noop(fn):
    @functools.wraps(fn)
    def wrapper(self, *args, **kwargs):
        if self.enabled:
            return fn(self, *args, **kwargs)
    return wrapper


class MailChimpError(Exception):
    pass


def _raise_response_error(r):
    # in case of a 500 error, the response might not be a JSON
    try:
        error_data = r.json()
    except ValueError:
        error_data = { "response": r }
    raise MailChimpError(error_data)


class MailChimpClient(object):
    """
    MailChimp class to communicate with the v3 API
    """
    def __init__(self, mc_api=None, mc_user='python-mailchimp', access_token=None, enabled=True, timeout=None,
                 request_hooks=None, request_headers=None):
        """
        Initialize the class with your optional user_id and required api_key.

        If `enabled` is not True, these methods become no-ops. This is
        particularly useful for testing or disabling with configuration.

        :param mc_api: Mailchimp API key
        :type mc_api: :py:class:`str`
        :param mc_user: Mailchimp user id
        :type mc_user: :py:class:`str`
        :param access_token: The OAuth access token
        :type access_token: :py:class:`str`
        :param enabled: Whether the API should execute any requests
        :type enabled: :py:class:`bool`
        :param timeout: (optional) How long to wait for the server to send
            data before giving up, as a float, or a :ref:`(connect timeout,
            read timeout) <timeouts>` tuple.
        :type timeout: float or tuple
        :param request_hooks: (optional) Hooks for :py:func:`requests.requests`.
        :type request_hooks: :py:class:`dict`
        :param request_headers: (optional) Headers for
            :py:func:`requests.requests`.
        :type request_headers: :py:class:`dict`
        """
        super(MailChimpClient, self).__init__()
        self.enabled = enabled
        self.timeout = timeout
        if access_token:
            self.auth = MailChimpOAuth(access_token)
            self.base_url = self.auth.get_base_url() + '/3.0/'
        elif mc_api:
            if not re.match(r"^[0-9a-f]{32}$", mc_api.split('-')[0]):
                raise ValueError('The API key that you have entered is not valid, did you enter a username by mistake?\n'
                                 'The order of arguments for API key and username has reversed in 2.1.0')
            self.auth = HTTPBasicAuth(mc_user, mc_api)
            datacenter = mc_api.split('-').pop()
            self.base_url = 'https://{0}.api.mailchimp.com/3.0/'.format(datacenter)
        else:
            raise Exception('You must provide an OAuth access token or API key')
        self.request_headers = request_headers or requests.utils.default_headers()
        self.request_hooks = request_hooks or requests.hooks.default_hooks()


    def _make_request(self, **kwargs):
        _logger.info(u'{method} Request: {url}'.format(**kwargs))
        if kwargs.get('json'):
            _logger.info('PAYLOAD: {json}'.format(**kwargs))

        response = requests.request(**kwargs)

        _logger.info(u'{method} Response: {status}'\
            .format(method=kwargs['method'], status=response.status_code))

        return response


    @_enabled_or_noop
    def _post(self, url, data=None):
        """
        Handle authenticated POST requests

        :param url: The url for the endpoint including path parameters
        :type url: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:data:`none` or :py:class:`dict`
        :returns: The JSON output from the API or an error message
        """
        url = urljoin(self.base_url, url)
        try:
            r = self._make_request(**dict(
                method='POST',
                url=url,
                json=data,
                auth=self.auth,
                timeout=self.timeout,
                hooks=self.request_hooks,
                headers=self.request_headers
            ))
        except requests.exceptions.RequestException as e:
            raise e
        else:
            if r.status_code >= 400:
                _raise_response_error(r)

            if r.status_code == 204:
                return None
            return r.json()


    @_enabled_or_noop
    def _get(self, url, **queryparams):
        """
        Handle authenticated GET requests

        :param url: The url for the endpoint including path parameters
        :type url: :py:class:`str`
        :param queryparams: The query string parameters
        :returns: The JSON output from the API
        """
        url = urljoin(self.base_url, url)
        if len(queryparams):
            url += '?' + urlencode(queryparams)
        try:
            r = self._make_request(**dict(
                method='GET',
                url=url,
                auth=self.auth,
                timeout=self.timeout,
                hooks=self.request_hooks,
                headers=self.request_headers
            ))
        except requests.exceptions.RequestException as e:
            raise e
        else:
            if r.status_code >= 400:
                _raise_response_error(r)
            return r.json()


    @_enabled_or_noop
    def _delete(self, url):
        """
        Handle authenticated DELETE requests

        :param url: The url for the endpoint including path parameters
        :type url: :py:class:`str`
        :returns: The JSON output from the API
        """
        url = urljoin(self.base_url, url)
        try:
            r = self._make_request(**dict(
                method='DELETE',
                url=url,
                auth=self.auth,
                timeout=self.timeout,
                hooks=self.request_hooks,
                headers=self.request_headers
            ))
        except requests.exceptions.RequestException as e:
            raise e
        else:
            if r.status_code >= 400:
                _raise_response_error(r)
            if r.status_code == 204:
                return
            return r.json()


    @_enabled_or_noop
    def _patch(self, url, data=None):
        """
        Handle authenticated PATCH requests

        :param url: The url for the endpoint including path parameters
        :type url: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:data:`none` or :py:class:`dict`
        :returns: The JSON output from the API
        """
        url = urljoin(self.base_url, url)
        try:
            r = self._make_request(**dict(
                method='PATCH',
                url=url,
                json=data,
                auth=self.auth,
                timeout=self.timeout,
                hooks=self.request_hooks,
                headers=self.request_headers
            ))
        except requests.exceptions.RequestException as e:
            raise e
        else:
            if r.status_code >= 400:
                _raise_response_error(r)
            return r.json()


    @_enabled_or_noop
    def _put(self, url, data=None):
        """
        Handle authenticated PUT requests

        :param url: The url for the endpoint including path parameters
        :type url: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:data:`none` or :py:class:`dict`
        :returns: The JSON output from the API
        """
        url = urljoin(self.base_url, url)
        try:
            r = self._make_request(**dict(
                method='PUT',
                url=url,
                json=data,
                auth=self.auth,
                timeout=self.timeout,
                hooks=self.request_hooks,
                headers=self.request_headers
            ))
        except requests.exceptions.RequestException as e:
            raise e
        else:
            if r.status_code >= 400:
                _raise_response_error(r)
            return r.json()


class MailChimpOAuth(requests.auth.AuthBase):
    """
    Authentication class for authentication with OAuth2. Acquiring an OAuth2
    for MailChimp can be done by following the instructions in the
    documentation found at
    http://developer.mailchimp.com/documentation/mailchimp/guides/how-to-use-oauth2/
    """
    def __init__(self, access_token):
        """
        Initialize the OAuth and save the access token

        :param access_token: The access token provided by OAuth authentication
        :type access_token: :py:class:`str`
        """
        self._access_token = access_token


    def __call__(self, r):
        """
        Authorize with the access token provided in __init__
        """
        r.headers['Authorization'] = 'OAuth ' + self._access_token
        return r


    def get_metadata(self):
        """
        Get the metadata returned after authentication
        """
        try:
            r = requests.get('https://login.mailchimp.com/oauth2/metadata', auth=self)
        except requests.exceptions.RequestException as e:
            raise e
        else:
            r.raise_for_status()
            output = r.json()
            if 'error' in output:
                raise requests.exceptions.RequestException(output['error'])
            return output


    def get_base_url(self):
        """
        Get the base_url from the authentication metadata
        """
        try:
            return self.get_metadata()['api_endpoint']
        except requests.exceptions.RequestException:
            raise



# coding=utf-8
"""
The List Segments API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/segments/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Segments/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.listsegmentmembers import ListSegmentMembers


class ListSegments(BaseApi):
    """
    Manage segments for a specific MailChimp list. A segment is a section of
    your list that includes only those subscribers who share specific common
    field information.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListSegments, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None
        self.segment_id = None
        self.members = ListSegmentMembers(self)


    def create(self, list_id, data):
        """
        Create a new segment in a specific list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "name": string*
        }
        """
        self.list_id = list_id
        if 'name' not in data:
            raise KeyError('The list segment must have a name')
        response = self._mc_client._post(url=self._build_path(list_id, 'segments'), data=data)
        if response is not None:
            self.segment_id = response['id']
        else:
            self.segment_id = None
        return response


    def all(self, list_id, get_all=False, **queryparams):
        """
        Get information about all available segments for a specific list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        queryparams['type'] = string
        queryparams['before_created_at'] = string
        queryparams['since_created_at'] = string
        queryparams['before_updated_at'] = string
        queryparams['since_updated_at'] = string
        """
        self.list_id = list_id
        self.segment_id = None
        if get_all:
            return self._iterate(url=self._build_path(list_id, 'segments'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(list_id, 'segments'), **queryparams)


    def get(self, list_id, segment_id, **queryparams):
        """
        Get information about a specific segment.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param segment_id: The unique id for the segment.
        :type segment_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.list_id = list_id
        self.segment_id = segment_id
        return self._mc_client._get(url=self._build_path(list_id, 'segments', segment_id), **queryparams)


    def update(self, list_id, segment_id, data):
        """
        Update a specific segment in a list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param segment_id: The unique id for the segment.
        :type segment_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "name": string*
        }
        """
        self.list_id = list_id
        self.segment_id = segment_id
        if 'name' not in data:
            raise KeyError('The list segment must have a name')
        return self._mc_client._patch(url=self._build_path(list_id, 'segments', segment_id), data=data)


    def update_members(self, list_id, segment_id, data):
        """
        Batch add/remove list members to static segment.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param segment_id: The unique id for the segment.
        :type segment_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "members_to_add": array,
            "members_to_remove": array
        }
        """
        self.list_id = list_id
        self.segment_id = segment_id
        return self._mc_client._post(url=self._build_path(list_id, 'segments', segment_id), data=data)


    def delete(self, list_id, segment_id):
        """
        Delete a specific segment in a list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param segment_id: The unique id for the segment.
        :type segment_id: :py:class:`str`
        """
        self.list_id = list_id
        self.segment_id = segment_id
        return self._mc_client._delete(url=self._build_path(list_id, 'segments', segment_id))



# coding=utf-8
"""
The Google Analytics API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/reports/google-analytics/
Schema: https://api.mailchimp.com/schema/3.0/Reports/GoogleAnalytics/Collection.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi

class ReportGoogleAnalytics(BaseApi):
    """
    Get top open locations for a specific campaign.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ReportGoogleAnalytics, self).__init__(*args, **kwargs)
        self.endpoint = 'reports'
        self.campaign_id = None
        self.profile_id = None

    def all(self, campaign_id, get_all=False, **queryparams):
        """
        Get a summary of Google Analytics reports for a specific campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams:  The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.campaign_id = campaign_id
        if get_all:
            return self._iterate(url=self._build_path(campaign_id, 'google-analytics'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(campaign_id, 'google-analytics'), **queryparams)

    def get(self, campaign_id, profile_id, **queryparams):
        """
        Get information about a specific Google Analytics report for a campaign.

        :param campaign_id: The unique id for the campaign
        :type campaign_id: :py:class:`str`
        :param profile_id: The Google Analytics View ID
        :type campaign_id: :py:class:`str`
        :param queryparams:
        :param queryparams:  The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.campaign_id = campaign_id
        self.profile_id = profile_id

        return self._mc_client._get(url=self._build_path(campaign_id, 'google-analytics', profile_id), **queryparams)






# coding=utf-8
"""
The List Member Notes API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/members/notes/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Members/Notes/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.helpers import check_subscriber_hash


class ListMemberNotes(BaseApi):
    """
    Retrieve recent notes for a specific list member.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListMemberNotes, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None
        self.subscriber_hash = None
        self.note_id = None


    def create(self, list_id, subscriber_hash, data):
        """
        Add a new note for a specific subscriber.

        The documentation lists only the note request body parameter so it is
        being documented and error-checked as if it were required based on the
        description of the method.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "note": string*
        }
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        if 'note' not in data:
            raise KeyError('The list member note must have a note')
        response = self._mc_client._post(url=self._build_path(list_id, 'members', subscriber_hash, 'notes'), data=data)
        if response is not None:
            self.note_id = response['id']
        else:
            self.note_id = None
        return response


    def all(self, list_id, subscriber_hash, get_all=False, **queryparams):
        """
        Get recent notes for a specific list member.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        self.note_id = None
        if get_all:
            return self._iterate(url=self._build_path(list_id, 'members', subscriber_hash, 'notes'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(list_id, 'members', subscriber_hash, 'notes'), **queryparams)


    def get(self, list_id, subscriber_hash, note_id, **queryparams):
        """
        Get a specific note for a specific list member.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        :param note_id: The id for the note.
        :type note_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        self.note_id = note_id
        return self._mc_client._get(
            url=self._build_path(list_id, 'members', subscriber_hash, 'notes', note_id),
            **queryparams
        )


    def update(self, list_id, subscriber_hash, note_id, data):
        """
        Update a specific note for a specific list member.

        The documentation lists only the note request body parameter so it is
        being documented and error-checked as if it were required based on the
        description of the method.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        :param note_id: The id for the note.
        :type note_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "note": string*
        }
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        self.note_id = note_id
        if 'note' not in data:
            raise KeyError('The list member note must have a note')
        return self._mc_client._patch(
            url=self._build_path(list_id, 'members', subscriber_hash, 'notes', note_id),
            data=data
        )


    def delete(self, list_id, subscriber_hash, note_id):
        """
        Delete a specific note for a specific list member.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        :param note_id: The id for the note.
        :type note_id: :py:class:`str`
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        self.note_id = note_id
        return self._mc_client._delete(url=self._build_path(list_id, 'members', subscriber_hash, 'notes', note_id))



# coding=utf-8
"""
The Campaigns API actions endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/campaigns/
Schema: https://api.mailchimp.com/schema/3.0/CampaignFolders/Instance.json
"""
from __future__ import unicode_literals

from datetime import timedelta

from mailchimp3.baseapi import BaseApi
from mailchimp3.helpers import check_email


class CampaignActions(BaseApi):
    """
    Campaigns are how you send emails to your MailChimp list. Use the
    Campaigns API calls to manage campaigns in your MailChimp account.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(CampaignActions, self).__init__(*args, **kwargs)
        self.endpoint = 'campaigns'
        self.campaign_id = None


    # Pro feature
    def cancel(self, campaign_id):
        """
        Cancel a Regular or Plain-Text Campaign after you send, before all of
        your recipients receive it. This feature is included with MailChimp
        Pro.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        """
        self.campaign_id = campaign_id
        return self._mc_client._post(url=self._build_path(campaign_id, 'actions/cancel-send'))


    def pause(self, campaign_id):
        """
        Pause an RSS-Driven campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        """
        self.campaign_id = campaign_id
        return self._mc_client._post(url=self._build_path(campaign_id, 'actions/pause'))


    def replicate(self, campaign_id):
        """
        Replicate a campaign in saved or send status.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        """
        self.campaign_id = campaign_id
        return self._mc_client._post(url=self._build_path(campaign_id, 'actions/replicate'))


    def resume(self, campaign_id):
        """
        Resume an RSS-Driven campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        """
        self.campaign_id = campaign_id
        return self._mc_client._post(url=self._build_path(campaign_id, 'actions/resume'))


    def schedule(self, campaign_id, data):
        """
        Schedule a campaign for delivery. If you’re using Multivariate
        Campaigns to test send times or sending RSS Campaigns, use the send
        action instead.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "schedule_time": datetime* (A UTC timezone datetime that ends on the quarter hour [:00, :15, :30, or :45])
        }
        """
        if not data['schedule_time']:
            raise ValueError('You must supply a schedule_time')
        else:
            if data['schedule_time'].tzinfo is None:
                raise ValueError('The schedule_time must be in UTC')
            else:
                if data['schedule_time'].tzinfo.utcoffset(None) != timedelta(0):
                    raise ValueError('The schedule_time must be in UTC')
        if data['schedule_time'].minute not in [0, 15, 30, 45]:
            raise ValueError('The schedule_time must end on the quarter hour (00, 15, 30, 45)')
        data['schedule_time'] = data['schedule_time'].strftime('%Y-%m-%dT%H:%M:00+00:00')
        self.campaign_id = campaign_id
        return self._mc_client._post(url=self._build_path(campaign_id, 'actions/schedule'), data=data)


    def send(self, campaign_id):
        """
        Send a MailChimp campaign. For RSS Campaigns, the campaign will send
        according to its schedule. All other campaigns will send immediately.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        """
        self.campaign_id = campaign_id
        return self._mc_client._post(url=self._build_path(campaign_id, 'actions/send'))


    def resend(self, campaign_id):
        """
        Creates a Resend to Non-Openers version of this campaign. We will also
        check if this campaign meets the criteria for Resend to Non-Openers
        campaigns.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        """
        self.campaign_id = campaign_id
        return self._mc_client._post(url=self._build_path(campaign_id, 'actions/create-resend'))


    def test(self, campaign_id, data):
        """
        Send a test email.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "test_emails": array*,
            "send_type": string* (Must be one of "html" or "plaintext")
        }
        """
        for email in data['test_emails']:
            check_email(email)
        if data['send_type'] not in ['html', 'plaintext']:
            raise ValueError('The send_type must be either "html" or "plaintext"')
        self.campaign_id = campaign_id
        return self._mc_client._post(url=self._build_path(campaign_id, 'actions/test'), data=data)


    def unschedule(self, campaign_id):
        """
        Unschedule a scheduled campaign that hasn’t started sending.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        """
        self.campaign_id = campaign_id
        return self._mc_client._post(url=self._build_path(campaign_id, 'actions/unschedule'))



# coding=utf-8
"""
Mailchimp v3 Api SDK

Documentation at http://developer.mailchimp.com/documentation/mailchimp/reference/overview/
"""
# API Client
from mailchimp3.mailchimpclient import MailChimpClient
# API Root
from mailchimp3.entities.root import Root
# Authorized Apps
from mailchimp3.entities.authorizedapps import AuthorizedApps
# Automations
from mailchimp3.entities.automations import Automations
from mailchimp3.entities.automationactions import AutomationActions
from mailchimp3.entities.automationemails import AutomationEmails
from mailchimp3.entities.automationemailactions import AutomationEmailActions
from mailchimp3.entities.automationemailqueues import AutomationEmailQueues
from mailchimp3.entities.automationremovedsubscribers import AutomationRemovedSubscribers
# Batch Operations
from mailchimp3.entities.batchoperations import BatchOperations
# Batch Webhooks
from mailchimp3.entities.batchwebhooks import BatchWebhooks
# Campaign Folders
from mailchimp3.entities.campaignfolders import CampaignFolders
# Campaigns
from mailchimp3.entities.campaigns import Campaigns
from mailchimp3.entities.campaignactions import CampaignActions
from mailchimp3.entities.campaigncontent import CampaignContent
from mailchimp3.entities.campaignfeedback import CampaignFeedback
from mailchimp3.entities.campaignsendchecklist import CampaignSendChecklist
# Conversations
from mailchimp3.entities.conversations import Conversations
from mailchimp3.entities.conversationmessages import ConversationMessages
# Customer Journeys
from mailchimp3.entities.customerjourney import CustomerJourney
# E-commerce Stores
from mailchimp3.entities.stores import Stores
from mailchimp3.entities.storecarts import StoreCarts
from mailchimp3.entities.storecartlines import StoreCartLines
from mailchimp3.entities.storecustomers import StoreCustomers
from mailchimp3.entities.storeorders import StoreOrders
from mailchimp3.entities.storeorderlines import StoreOrderLines
from mailchimp3.entities.storeproducts import StoreProducts
from mailchimp3.entities.storeproductimages import StoreProductImages
from mailchimp3.entities.storeproductvariants import StoreProductVariants
from mailchimp3.entities.storepromorules import StorePromoRules
from mailchimp3.entities.storepromocodes import StorePromoCodes
# File Manager Files
from mailchimp3.entities.filemanagerfiles import FileManagerFiles
# File Manager Folders
from mailchimp3.entities.filemanagerfolders import FileManagerFolders
# Landinge Pages
from mailchimp3.entities.landingpages import LandingPages
from mailchimp3.entities.landingpageaction import LandingPageAction
from mailchimp3.entities.landingpagecontent import LandingPageContent
# Lists
from mailchimp3.entities.lists import Lists
from mailchimp3.entities.listabusereports import ListAbuseReports
from mailchimp3.entities.listactivity import ListActivity
from mailchimp3.entities.listclients import ListClients
from mailchimp3.entities.listgrowthhistory import ListGrowthHistory
from mailchimp3.entities.listinterestcategories import ListInterestCategories
from mailchimp3.entities.listinterestcategoryinterest import ListInterestCategoryInterest
from mailchimp3.entities.listmembers import ListMembers
from mailchimp3.entities.listmemberactivity import ListMemberActivity
from mailchimp3.entities.listmemberevents import ListMemberEvents
from mailchimp3.entities.listmembergoals import ListMemberGoals
from mailchimp3.entities.listmembernotes import ListMemberNotes
from mailchimp3.entities.listmembertags import ListMemberTags
from mailchimp3.entities.listmergefields import ListMergeFields
from mailchimp3.entities.listsegments import ListSegments
from mailchimp3.entities.listsegmentmembers import ListSegmentMembers
from mailchimp3.entities.listsignupforms import ListSignupForms
from mailchimp3.entities.listwebhooks import ListWebhooks

# ping
from mailchimp3.entities.ping import Ping

# Reports
from mailchimp3.entities.reports import Reports
from mailchimp3.entities.reportcampaignabusereports import ReportCampaignAbuseReports
from mailchimp3.entities.reportcampaignadvice import ReportCampaignAdvice
from mailchimp3.entities.reportclickdetailreports import ReportClickDetailReports
from mailchimp3.entities.reportclickdetailmembers import ReportClickDetailMembers
from mailchimp3.entities.reportdomainperformance import ReportDomainPerformance
from mailchimp3.entities.reporteepurl import ReportEepURL
from mailchimp3.entities.reportemailactivity import ReportEmailActivity
from mailchimp3.entities.reportlocations import ReportLocations
from mailchimp3.entities.reportsentto import ReportSentTo
from mailchimp3.entities.reportsubreports import ReportSubReports
from mailchimp3.entities.reportunsubscribes import ReportUnsubscribes
from mailchimp3.entities.reportopendetails import ReportOpenDetails
from mailchimp3.entities.reportgoogleanalytics import ReportGoogleAnalytics

# Search Campaigns
from mailchimp3.entities.searchcampaigns import SearchCampaigns
# Search Members
from mailchimp3.entities.searchmembers import SearchMembers
# Template Folders
from mailchimp3.entities.templatefolders import TemplateFolders
# Templates
from mailchimp3.entities.templates import Templates
from mailchimp3.entities.templatedefaultcontent import TemplateDefaultContent


class MailChimp(MailChimpClient):
    """
    MailChimp class to communicate with the v3 API
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the class with your api_key and user_id and attach all of
        the endpoints
        """
        super(MailChimp, self).__init__(*args, **kwargs)
        # API Root
        self.root = self.api_root = Root(self)
        # Authorized Apps
        self.authorized_apps = AuthorizedApps(self)
        # Automations - Paid feature
        self.automations = Automations(self)
        self.automations.actions = AutomationActions(self)
        self.automations.emails = AutomationEmails(self)
        self.automations.emails.actions = AutomationEmailActions(self)
        self.automations.emails.queues = AutomationEmailQueues(self)
        self.automations.removed_subscribers = AutomationRemovedSubscribers(self)
        # Batch Operations
        self.batches = self.batch_operations = BatchOperations(self)
        # Batch Webhooks
        self.batch_webhooks = BatchWebhooks(self)
        # Campaign Folders
        self.campaign_folders = CampaignFolders(self)
        # Campaigns
        self.campaigns = Campaigns(self)
        self.campaigns.actions = CampaignActions(self)
        self.campaigns.content = CampaignContent(self)
        self.campaigns.feedback = CampaignFeedback(self)
        self.campaigns.send_checklist = CampaignSendChecklist(self)
        # Conversations - Paid feature
        self.conversations = Conversations(self)
        self.conversations.messages = ConversationMessages(self)
        # Customer Journey
        self.journeys = self.customer_journeys = CustomerJourney(self)
        # E-commerce Stores
        self.stores = self.ecommerce = Stores(self)
        self.stores.carts = StoreCarts(self)
        self.stores.carts.lines = StoreCartLines(self)
        self.stores.customers = StoreCustomers(self)
        self.stores.orders = StoreOrders(self)
        self.stores.orders.lines = StoreOrderLines(self)
        self.stores.products = StoreProducts(self)
        self.stores.products.images = StoreProductImages(self)
        self.stores.products.variants = StoreProductVariants(self)
        self.stores.promo_rules = StorePromoRules(self)
        self.stores.promo_codes = StorePromoCodes(self)
        # File Manager Files
        self.files = FileManagerFiles(self)
        # File Manager Folders
        self.folders = FileManagerFolders(self)
        # Landinge Pages
        self.landing_pages = LandingPages(self)
        self.landing_pages.actions = LandingPageAction(self)
        self.landing_pages.content = LandingPageContent(self)
        # Lists
        self.lists = Lists(self)
        self.lists.abuse_reports = ListAbuseReports(self)
        self.lists.activity = ListActivity(self)
        self.lists.clients = ListClients(self)
        self.lists.growth_history = ListGrowthHistory(self)
        self.lists.interest_categories = ListInterestCategories(self)
        self.lists.interest_categories.interests = ListInterestCategoryInterest(self)
        self.lists.members = ListMembers(self)
        self.lists.members.activity = ListMemberActivity(self)
        self.lists.members.events = ListMemberEvents(self)
        self.lists.members.goals = ListMemberGoals(self)
        self.lists.members.notes = ListMemberNotes(self)
        self.lists.members.tags = ListMemberTags(self)
        self.lists.merge_fields = ListMergeFields(self)
        self.lists.segments = ListSegments(self)
        self.lists.segments.members = ListSegmentMembers(self)
        self.lists.signup_forms = ListSignupForms(self)
        self.lists.webhooks = ListWebhooks(self)
        # Ping
        self.ping = Ping(self)
        # Reports
        self.reports = Reports(self)
        self.reports.abuse_reports = ReportCampaignAbuseReports(self)
        self.reports.advice = ReportCampaignAdvice(self)
        self.reports.click_details = ReportClickDetailReports(self)
        self.reports.click_details.members = ReportClickDetailMembers(self)
        self.reports.domain_performance = ReportDomainPerformance(self)
        self.reports.eepurl = ReportEepURL(self)
        self.reports.email_activity = ReportEmailActivity(self)
        self.reports.locations = ReportLocations(self)
        self.reports.sent_to = ReportSentTo(self)
        self.reports.subreports = ReportSubReports(self)
        self.reports.unsubscribes = ReportUnsubscribes(self)
        self.reports.open_details = ReportOpenDetails(self)
        self.reports.google_analytics = ReportGoogleAnalytics(self)
        # Search Campaigns
        self.search_campaigns = SearchCampaigns(self)
        # Search Members
        self.search_members = SearchMembers(self)
        # Template Folders
        self.template_folders = TemplateFolders(self)
        # Templates
        self.templates = Templates(self)
        self.templates.default_content = TemplateDefaultContent(self)



# coding=utf-8
"""
The Landinge Page actions API endpoint

Documentation: https://mailchimp.com/developer/reference/landing-pages/
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class LandingPageAction(BaseApi):
    """
    Manage your Landing Pages, including publishing and unpublishing.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(LandingPageAction, self).__init__(*args, **kwargs)
        self.endpoint = 'landing-pages'
        self.page_id = None


    def publish(self, page_id):
        """
        Publish a landing page that is in draft, unpublished, or has been
        previously published and edited.

        :param page_id: The unique id for the page.
        :type page_id: :py:class:`str`
        """
        self.page_id = page_id
        return self._mc_client._post(url=self._build_path(page_id, 'actions/publish'))


    def unpublish(self, page_id):
        """
        Unpublish a landing page that is in draft or has been published.

        :param page_id: The unique id for the page.
        :type page_id: :py:class:`str`
        """
        self.page_id = page_id
        return self._mc_client._post(url=self._build_path(page_id, 'actions/unpublish'))



# coding=utf-8
"""
The Customer Journey API endpoint

Documentation: https://mailchimp.com/developer/marketing/api/customer-journeys-journeys-steps-actions/
Schema: https://us1.api.mailchimp.com/schema/3.0/Paths/CustomerJourneys/Journeys/Steps/Actions/Trigger.json
"""

from mailchimp3.baseapi import BaseApi
from mailchimp3.helpers import check_email


class CustomerJourney(BaseApi):
    """
    Manage specific customer journeys in your Mailchimp account.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(CustomerJourney, self).__init__(*args, **kwargs)
        self.endpoint = 'customer-journeys'
        self.journey_id = None
        self.step_id = None

    def trigger(self, journey_id, step_id, data):
        """
        Trigger a step in a customer journey.

        :param journey_id: The unique id for the Customer Journey
        :type journey_id: :py:class:`str`
        :param step_id: The unique id for the step within the customer journey
        :type step_id: :py:class:`str`
        :type data: :py:class:`dict`
        data = {
            "email": string*,
        }
        """
        self.journey_id = journey_id
        self.step_id = step_id
        if 'email_address' not in data:
            raise KeyError('The automation email queue must have an email_address')

        check_email(data['email_address'])
        response = self._mc_client._post(
            url=self._build_path("journeys", journey_id, 'steps', step_id, 'actions', "trigger"),
            data=data
        )

        return response



# coding=utf-8
"""
The Campaign Feedback API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/campaigns/feedback/
Schema: https://api.mailchimp.com/schema/3.0/Campaigns/Feedback/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class CampaignFeedback(BaseApi):
    """
    Post comments, reply to team feedback, and send test emails while you’re
    working together on a MailChimp campaign.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(CampaignFeedback, self).__init__(*args, **kwargs)
        self.endpoint = 'campaigns'
        self.campaign_id = None
        self.feedback_id = None


    def create(self, campaign_id, data, **queryparams):
        """
        Add feedback on a specific campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "message": string*
        }
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.campaign_id = campaign_id
        if 'message' not in data:
            raise KeyError('The campaign feedback must have a message')
        response = self._mc_client._post(url=self._build_path(campaign_id, 'feedback'), data=data, **queryparams)
        if response is not None:
            self.feedback_id = response['feedback_id']
        else:
            self.feedback_id = None
        return response


    def all(self, campaign_id, get_all=False, **queryparams):
        """
        Get team feedback while you’re working together on a MailChimp
        campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.campaign_id = campaign_id
        self.feedback_id = None
        if get_all:
            return self._iterate(url=self._build_path(campaign_id, 'feedback'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(campaign_id, 'feedback'), **queryparams)


    def get(self, campaign_id, feedback_id, **queryparams):
        """
        Get a specific feedback message from a campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param feedback_id: The unique id for the feedback message.
        :type feedback_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.campaign_id = campaign_id
        self.feedback_id = feedback_id
        return self._mc_client._get(url=self._build_path(campaign_id, 'feedback', feedback_id), **queryparams)


    def update(self, campaign_id, feedback_id, data):
        """
        Update a specific feedback message for a campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param feedback_id: The unique id for the feedback message.
        :type feedback_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "message": string*
        }
        """
        self.campaign_id = campaign_id
        self.feedback_id = feedback_id
        if 'message' not in data:
            raise KeyError('The campaign feedback must have a message')
        return self._mc_client._patch(url=self._build_path(campaign_id, 'feedback', feedback_id), data=data)


    def delete(self, campaign_id, feedback_id):
        """
        Remove a specific feedback message for a campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param feedback_id: The unique id for the feedback message.
        :type feedback_id: :py:class:`str`
        """
        self.campaign_id = campaign_id
        self.feedback_id = feedback_id
        return self._mc_client._delete(url=self._build_path(campaign_id, 'feedback', feedback_id))



# coding=utf-8
"""
The E-commerce Store Customers API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/ecommerce/stores/customers/
Schema: https://api.mailchimp.com/schema/3.0/Ecommerce/Stores/Customers/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.helpers import check_email


class StoreCustomers(BaseApi):
    """
    Add Customers to your Store to track their orders and to view E-Commerce
    Data for your MailChimp lists and campaigns. Each Customer is connected to
    a MailChimp list member, so adding a Customer can also add or update a
    list member.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(StoreCustomers, self).__init__(*args, **kwargs)
        self.endpoint = 'ecommerce/stores'
        self.store_id = None
        self.customer_id = None


    def create(self, store_id, data):
        """
        Add a new customer to a store.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "id": string*,
            "email_address": string*,
            "opt_in_status": boolean*
        }
        """
        self.store_id = store_id
        if 'id' not in data:
            raise KeyError('The store customer must have an id')
        if 'email_address' not in data:
            raise KeyError('The store customer must have an email_address')
        check_email(data['email_address'])
        if 'opt_in_status' not in data:
            raise KeyError('The store customer must have an opt_in_status')
        if data['opt_in_status'] not in [True, False]:
            raise TypeError('The opt_in_status must be True or False')
        response = self._mc_client._post(url=self._build_path(store_id, 'customers'), data=data)
        if response is not None:
            self.customer_id = response['id']
        else:
            self.customer_id = None
        return response


    def all(self, store_id, get_all=False, **queryparams):
        """
        Get information about a store’s customers.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        queryparams['email_address'] = string
        """
        self.store_id = store_id
        self.customer_id = None
        if get_all:
            return self._iterate(url=self._build_path(store_id, 'customers'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(store_id, 'customers'), **queryparams)


    def get(self, store_id, customer_id, **queryparams):
        """
        Get information about a specific customer.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param customer_id: The id for the customer of a store.
        :type customer_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.store_id = store_id
        self.customer_id = customer_id
        return self._mc_client._get(url=self._build_path(store_id, 'customers', customer_id), **queryparams)


    def update(self, store_id, customer_id, data):
        """
        Update a customer.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param customer_id: The id for the customer of a store.
        :type customer_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        """
        self.store_id = store_id
        self.customer_id = customer_id
        return self._mc_client._patch(url=self._build_path(store_id, 'customers', customer_id), data=data)


    def create_or_update(self, store_id, customer_id, data):
        """
        Add or update a customer.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param customer_id: The id for the customer of a store.
        :type customer_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "id": string*,
            "email_address": string*,
            "opt_in_status": boolean
        }
        """
        self.store_id = store_id
        self.customer_id = customer_id
        if 'id' not in data:
            raise KeyError('The store customer must have an id')
        if 'email_address' not in data:
            raise KeyError('Each store customer must have an email_address')
        check_email(data['email_address'])
        if 'opt_in_status' not in data:
            raise KeyError('The store customer must have an opt_in_status')
        if data['opt_in_status'] not in [True, False]:
            raise TypeError('The opt_in_status must be True or False')
        return self._mc_client._put(url=self._build_path(store_id, 'customers', customer_id), data=data)


    def delete(self, store_id, customer_id):
        """
        Delete a customer from a store.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param customer_id: The id for the customer of a store.
        :type customer_id: :py:class:`str`
        """
        self.store_id = store_id
        self.customer_id = customer_id
        return self._mc_client._delete(url=self._build_path(store_id, 'customers', customer_id))



# coding=utf-8
"""
The Conversations API endpoint

Note: This is a paid feature

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/conversations/
Schema: https://api.mailchimp.com/schema/3.0/Conversations/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.conversationmessages import ConversationMessages


class Conversations(BaseApi):
    """
    Conversation tracking is a paid feature that lets you view subscribers’
    replies to your campaigns in your MailChimp account.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(Conversations, self).__init__(*args, **kwargs)
        self.endpoint = 'conversations'
        self.conversation_id = None
        self.messages = ConversationMessages(self)


    # Paid feature
    def all(self, get_all=False, **queryparams):
        """
        Get a list of conversations for the account.

        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        queryparams['has_unread_messages'] = string
        queryparams['list_id'] = string
        queryparams['campaign_id'] = string
        """
        self.conversation_id = None
        if get_all:
            return self._iterate(url=self._build_path(), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(), **queryparams)


    # Paid feature
    def get(self, conversation_id, **queryparams):
        """
        Get details about an individual conversation.

        :param conversation_id: The unique id for the conversation.
        :type conversation_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.conversation_id = conversation_id
        return self._mc_client._get(url=self._build_path(conversation_id), **queryparams)



# coding=utf-8
"""
The List Clients API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/clients/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Clients/Collection.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class ListClients(BaseApi):
    """
    Get information about the most popular email clients for subscribers in a
    specific MailChimp list.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListClients, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None


    def all(self, list_id, **queryparams):
        """
        Get a list of the top email clients based on user-agent strings.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.list_id = list_id
        return self._mc_client._get(url=self._build_path(list_id, 'clients'), **queryparams)



# coding=utf-8
"""
The Location Report API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/reports/locations/
Schema: https://api.mailchimp.com/schema/3.0/Reports/Locations/Collection.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class ReportLocations(BaseApi):
    """
    Get top open locations for a specific campaign.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ReportLocations, self).__init__(*args, **kwargs)
        self.endpoint = 'reports'
        self.campaign_id = None


    def all(self, campaign_id, get_all=False, **queryparams):
        """
        Get top open locations for a specific campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.campaign_id = campaign_id
        if get_all:
            return self._iterate(url=self._build_path(campaign_id, 'locations'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(campaign_id, 'locations'), **queryparams)



# coding=utf-8
"""
The List Signup Forms API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/signup-forms/
Schema: https://api.mailchimp.com/schema/3.0/Lists/SignupForms/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class ListSignupForms(BaseApi):
    """
    Manage list signup forms.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListSignupForms, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None


    def create(self, list_id, data):
        """
        Create a customized list signup form.

        No fields are listed as required in the documentation and the
        description of the method does not indicate any required fields
        either.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        """
        self.list_id = list_id
        response = self._mc_client._post(url=self._build_path(list_id, 'signup-forms'), data=data)
        return response


    def all(self, list_id):
        """
        Get signup forms for a specific list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        """
        self.list_id = list_id
        return self._mc_client._get(url=self._build_path(list_id, 'signup-forms'))



# coding=utf-8
"""
The Landing Pages API endpoint

Documentation: https://mailchimp.com/developer/reference/landing-pages/
Schema: https://api.mailchimp.com/schema/3.0/LandingPages/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.landingpageaction import LandingPageAction
from mailchimp3.entities.landingpagecontent import LandingPageContent


class LandingPages(BaseApi):
    """
    Manage your Landing Pages, including publishing and unpublishing.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(LandingPages, self).__init__(*args, **kwargs)
        self.endpoint = 'landing-pages'
        self.page_id = None
        self.content = LandingPageContent(self)
        self.actions = LandingPageAction(self)


    def create(self, data):
        """
        Create a new Landing Page.

        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "list_id": string*,
        }
        """
        if 'list_id' not in data:
            raise KeyError('The landing page must have a list_id')
        response = self._mc_client._post(url=self._build_path(), data=data)
        if response is not None:
            self.page_id = response['id']
        else:
            self.page_id = None
        return response


    def all(self, get_all=False, **queryparams):
        """
        Get all landinge pages.

        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['sort_field'] = string
        queryparams['sort_dir'] = string
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        """
        self.page_id = None
        if get_all:
            return self._iterate(url=self._build_path(), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(), **queryparams)


    def get(self, page_id, **queryparams):
        """
        Get information about a specific page.

        :param page_id: The unique id for the page.
        :type page_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.page_id = page_id
        return self._mc_client._get(url=self._build_path(page_id), **queryparams)


    def update(self, page_id, data):
        """
        Update a landing page.

        :param page_id: The unique id for the landing page.
        :type page_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        """
        self.page_id = page_id
        return self._mc_client._patch(url=self._build_path(page_id), data=data)


    def delete(self, page_id):
        """
        Remove a landing page from your MailChimp account.

        :param page_id: The unique id for the landing page.
        :type page_id: :py:class:`str`
        """
        self.page_id = page_id
        return self._mc_client._delete(url=self._build_path(page_id))



# coding=utf-8
"""
The Automation Emails endpoint

Note: This is a paid feature

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/automations/emails/
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class AutomationEmailActions(BaseApi):
    """
    Manage individual emails in an Automation workflow.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(AutomationEmailActions, self).__init__(*args, **kwargs)
        self.endpoint = 'automations'
        self.workflow_id = None
        self.email_id = None


    # Paid feature
    def pause(self, workflow_id, email_id):
        """
        Pause an automated email.

        :param workflow_id: The unique id for the Automation workflow.
        :type workflow_id: :py:class:`str`
        :param email_id: The unique id for the Automation workflow email.
        :type email_id: :py:class:`str`
        """
        self.workflow_id = workflow_id
        self.email_id = email_id
        return self._mc_client._post(url=self._build_path(workflow_id, 'emails', email_id, 'actions/pause'))


    # Paid feature
    def start(self, workflow_id, email_id):
        """
        Start an automated email.

        :param workflow_id: The unique id for the Automation workflow.
        :type workflow_id: :py:class:`str`
        :param email_id: The unique id for the Automation workflow email.
        :type email_id: :py:class:`str`
        """
        self.workflow_id = workflow_id
        self.email_id = email_id
        return self._mc_client._post(url=self._build_path(workflow_id, 'emails', email_id, 'actions/start'))

    #Paid feature
    def delete(self, workflow_id, email_id):
        """
        Removes an individual Automation workflow email.

        :param workflow_id: The unique id for the Automation workflow.
        :type workflow_id: :py:class:`str`
        :param email_id: The unique id for the Automation workflow email.
        :type email_id: :py:class:`str`
        """

        self.workflow_id = workflow_id
        self.email_id = email_id
        return self._mc_client._delete(url=self._build_path(workflow_id, 'emails', email_id))





# coding=utf-8
"""
The Templates API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/templates/
Schema: https://api.mailchimp.com/schema/3.0/Templates/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.templatedefaultcontent import TemplateDefaultContent


class Templates(BaseApi):
    """
    Manage your MailChimp templates. A template is an HTML file used to create
    the layout and basic design for a campaign.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(Templates, self).__init__(*args, **kwargs)
        self.endpoint = 'templates'
        self.template_id = None
        self.default_content = TemplateDefaultContent(self)


    def create(self, data):
        """
        Create a new template for the account. Only Classic templates are
        supported.

        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "name": string*,
            "html": string*
        }
        """
        if 'name' not in data:
            raise KeyError('The template must have a name')
        if 'html' not in data:
            raise KeyError('The template must have html')
        response = self._mc_client._post(url=self._build_path(), data=data)
        if response is not None:
            self.template_id = response['id']
        else:
            self.template_id = None
        return response


    def all(self, get_all=False, **queryparams):
        """
        Get a list of an account’s available templates.

        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        queryparams['created_by'] = string
        queryparams['before_created_at'] = string
        queryparams['since_created_at'] = string
        queryparams['type'] = string
        queryparams['folder_id'] = string
        """
        self.template_id = None
        if get_all:
            return self._iterate(url=self._build_path(), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(), **queryparams)


    def get(self, template_id, **queryparams):
        """
        Get information about a specific template.

        :param template_id: The unique id for the template.
        :type template_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.template_id = template_id
        return self._mc_client._get(url=self._build_path(template_id), **queryparams)


    def update(self, template_id, data):
        """
        Update the name, HTML, or folder_id of an existing template.

        :param template_id: The unique id for the template.
        :type template_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "name": string*,
            "html": string*
        }
        """
        if 'name' not in data:
            raise KeyError('The template must have a name')
        if 'html' not in data:
            raise KeyError('The template must have html')
        self.template_id = template_id
        return self._mc_client._patch(url=self._build_path(template_id), data=data)


    def delete(self, template_id):
        """
        Delete a specific template.

        :param template_id: The unique id for the template.
        :type template_id: :py:class:`str`
        """
        self.template_id = template_id
        return self._mc_client._delete(url=self._build_path(template_id))



# coding=utf-8
"""
The Automation Emails endpoint

Note: This is a paid feature

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/automations/emails/
Schema: https://api.mailchimp.com/schema/3.0/Automations/Emails/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.automationemailactions import AutomationEmailActions
from mailchimp3.entities.automationemailqueues import AutomationEmailQueues


class AutomationEmails(BaseApi):
    """
    Manage individual emails in an Automation workflow.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(AutomationEmails, self).__init__(*args, **kwargs)
        self.endpoint = 'automations'
        self.workflow_id = None
        self.email_id = None
        self.actions = AutomationEmailActions(self)
        self.queues = AutomationEmailQueues(self)


    # Paid feature
    def all(self, workflow_id, get_all=False, **queryparams):
        """
        Get a summary of the emails in an Automation workflow.

        :param workflow_id: The unique id for the Automation workflow.
        :type workflow_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: the query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.workflow_id = workflow_id
        self.email_id = None
        if get_all:
            return self._iterate(url=self._build_path(workflow_id, 'emails'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(workflow_id, 'emails'), **queryparams)


    # Paid feature
    def get(self, workflow_id, email_id):
        """
        Get information about an individual Automation workflow email.

        :param workflow_id: The unique id for the Automation workflow.
        :type workflow_id: :py:class:`str`
        :param email_id: The unique id for the Automation workflow email.
        :type email_id: :py:class:`str`
        """
        self.workflow_id = workflow_id
        self.email_id = email_id
        return self._mc_client._get(url=self._build_path(workflow_id, 'emails', email_id))



# coding=utf-8
"""
The Landing Page Content API endpoint

Documentation: https://mailchimp.com/developer/reference/landing-pages/landing-pages_content/
Schema: https://api.mailchimp.com/schema/3.0/Definitions/LandingPages/Content/Response.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class LandingPageContent(BaseApi):
    """
    The HTML content for your Mailchimp landing pages.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(LandingPageContent, self).__init__(*args, **kwargs)
        self.endpoint = 'landing-pages'
        self.page_id = None


    def get(self, page_id, **queryparams):
        """
        Get the the HTML for your landing page.

        :param page_id: The unique id for the page.
        :type page_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.page_id = page_id
        return self._mc_client._get(url=self._build_path(page_id, 'content'), **queryparams)



# coding=utf-8
"""
The List Member Goals API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/members/goals/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Members/Goals/Collection.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.helpers import check_subscriber_hash


class ListMemberGoals(BaseApi):
    """
    Get information about recent goal events for a specific list member.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListMemberGoals, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None
        self.subscriber_hash = None


    def all(self, list_id, subscriber_hash, **queryparams):
        """
        Get the last 50 Goal events for a member on a specific list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        return self._mc_client._get(url=self._build_path(list_id, 'members', subscriber_hash, 'goals'), **queryparams)



from __future__ import unicode_literals
from ..baseapi import BaseApi


class Segments(BaseApi):

    def __init__(self, *args, **kwargs):
        super(Segments, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'

    def all(self, list_id, **queryparams):
        """
        returns the first 10 segments for a specific list.
        """
        return self._mc_client._get(url=self._build_path(list_id, 'segments'), **queryparams)

    def get(self, list_id, segment_id):
        """
        returns the specified list segment.
        """
        return self._mc_client._get(url=self._build_path(list_id, 'segments', segment_id))

    def update(self, list_id, segment_id, data):
        """
        updates an existing list segment.
        """
        return self._mc_client._patch(url=self._build_path(list_id, 'segments', segment_id), data=data)

    def delete(self, list_id, segment_id):
        """
        removes an existing list segment from the list. This cannot be undone.
        """
        return self._mc_client._delete(url=self._build_path(list_id, 'segments', segment_id))

    def create(self, list_id, data):
        """
        adds a new segment to the list.
        """
        return self._mc_client._post(url=self._build_path(list_id, 'segments'), data=data)



# coding=utf-8
"""
The List Abuse Reports API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/abuse-reports/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Abuse/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class ListAbuseReports(BaseApi):
    """
    Manage abuse complaints for a specific list. An abuse complaint occurs
    when your recipient reports an email as spam in their mail program.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListAbuseReports, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None
        self.report_id = None


    def all(self, list_id, get_all=False, **queryparams):
        """
        Get all abuse reports for a specific list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.list_id = list_id
        self.report_id = None
        if get_all:
            return self._iterate(url=self._build_path(list_id, 'abuse-reports'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(list_id, 'abuse-reports'), **queryparams)


    def get(self, list_id, report_id, **queryparams):
        """
        Get details about a specific abuse report.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param report_id: The id for the abuse report.
        :type report_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.list_id = list_id
        self.report_id = report_id
        return self._mc_client._get(url=self._build_path(list_id, 'abuse-reports', report_id), **queryparams)



# coding=utf-8
"""
The List Member Tags API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/members/tags/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Members/Tags/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.helpers import check_subscriber_hash


class ListMemberTags(BaseApi):
    """
    Retrieve tags for a specific list member.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListMemberTags, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None
        self.subscriber_hash = None


    def update(self, list_id, subscriber_hash, data):
        """
        Update tags for a specific subscriber.

        The documentation lists only the tags request body parameter so it is
        being documented and error-checked as if it were required based on the
        description of the method.

        The data list needs to include a "status" key. This determines if the
        tag should be added or removed from the user:

        data = {
            'tags': [
                {'name': 'foo', 'status': 'active'},
                {'name': 'bar', 'status': 'inactive'}
            ]
        }

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "tags": list*
        }
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        if 'tags' not in data:
            raise KeyError('The list member tags must have a tag')
        response = self._mc_client._post(url=self._build_path(list_id, 'members', subscriber_hash, 'tags'), data=data)
        return response


    def all(self, list_id, subscriber_hash, **queryparams):
        """
        Get all tags for a specific list member.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.subscriber_hash = subscriber_hash
        return self._mc_client._get(url=self._build_path(list_id, 'members', subscriber_hash, 'tags'), **queryparams)



# coding=utf-8
"""
The Campaign Advice Report API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/reports/advice/
Schema: https://api.mailchimp.com/schema/3.0/Reports/Advice/Collection.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class ReportCampaignAdvice(BaseApi):
    """
    Get recent feedback based on a campaign’s statistics.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ReportCampaignAdvice, self).__init__(*args, **kwargs)
        self.endpoint = 'reports'
        self.campaign_id = None


    def all(self, campaign_id, **queryparams):
        """
        Get feedback based on a campaign’s statistics. Advice feedback is
        based on campaign stats like opens, clicks, unsubscribes, bounces, and
        more.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.campaign_id = campaign_id
        return self._mc_client._get(url=self._build_path(campaign_id, 'advice'), **queryparams)



# coding=utf-8
"""
The Google Analytics API Endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/ping
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi

class Ping(BaseApi):
    """
    A health check endpoint for MailChimp API 3.0.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(Ping, self).__init__(*args, **kwargs)
        self.endpoint = 'ping'

    def get(self):
        """
        A health check for the API that won’t return any account-specific information.
        """
        return self._mc_client._get(url=self._build_path())



# coding=utf-8
"""
The File Manager Files API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/file-manager/files/
Schema: https://api.mailchimp.com/schema/3.0/FileManager/Files/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class FileManagerFiles(BaseApi):
    """
    Manage specific files in the File Manager for your MailChimp account. The
    File Manager is a place to store images, documents, and other files you
    include or link to in your campaigns, templates, or signup forms.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(FileManagerFiles, self).__init__(*args, **kwargs)
        self.endpoint = 'file-manager/files'
        self.file_id = None


    def create(self, data):
        """
        Upload a new image or file to the File Manager.

        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "name": string*,
            "file_data": string*
        }
        """
        if 'name' not in data:
            raise KeyError('The file must have a name')
        if 'file_data' not in data:
            raise KeyError('The file must have file_data')
        response = self._mc_client._post(url=self._build_path(), data=data)
        if response is not None:
            self.file_id = response['id']
        else:
            self.file_id = None
        return response


    def all(self, get_all=False, **queryparams):
        """
        Get a list of available images and files stored in the File Manager for the account.

        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        queryparams['type'] = string
        queryparams['created_by'] = string
        queryparams['before_created_at'] = string
        queryparams['since_created_at'] = string
        queryparams['sort_field'] = string
        queryparams['sort_dir'] = string
        """
        self.file_id = None
        if get_all:
            return self._iterate(url=self._build_path(), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(), **queryparams)


    def get(self, file_id, **queryparams):
        """
        Get information about a specific file in the File Manager.

        :param file_id: The unique id for the File Manager file.
        :type file_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.file_id = file_id
        return self._mc_client._get(url=self._build_path(file_id), **queryparams)


    def update(self, file_id, data):
        """
        Update a file in the File Manager.

        :param file_id: The unique id for the File Manager file.
        :type file_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "name": string*,
            "file_data": string*
        }
        """
        self.file_id = file_id
        if 'name' not in data:
            raise KeyError('The file must have a name')
        if 'file_data' not in data:
            raise KeyError('The file must have file_data')
        return self._mc_client._patch(url=self._build_path(file_id), data=data)


    def delete(self, file_id):
        """
        Remove a specific file from the File Manager.

        :param file_id: The unique id for the File Manager file.
        :type file_id: :py:class:`str`
        """
        self.file_id = file_id
        return self._mc_client._delete(url=self._build_path(file_id))



# coding=utf-8
"""
The E-commerce Store Orders endpoint API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/ecommerce/stores/orders/
Schema: https://api.mailchimp.com/schema/3.0/Ecommerce/Stores/Customers/Instance.json
"""
from __future__ import unicode_literals

import re

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.storeorderlines import StoreOrderLines


class StoreOrders(BaseApi):
    """
    Orders represent successful e-commerce transactions, and this data can be
    used to provide more detailed campaign reports, track sales, and
    personalize emails to your targeted consumers, and view other e-commerce
    data in your MailChimp account.

    .. note::
        If a financial_status or fulfillment_status value is provided when
        creating or updating an order, it will trigger a notification email if
        they have been designed and enabled in your account. The following sets
        of statuses trigger the following notification types:

        financial_status = 'paid' -> Order Invoice
        financial_status = 'pending' -> Order Confirmation
        financial_status = 'refunded' -> Refund Confirmation
        financial_status = 'cancelled' -> Cancellation Confirmation
        fulfillment_status = 'shipped' -> Shipping Confirmation

        The current list of notification types and triggers can be found at
        http://developer.mailchimp.com/documentation/mailchimp/guides/getting-started-with-ecommerce/#order-notifications
        and should be consulted in the event of any changes
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(StoreOrders, self).__init__(*args, **kwargs)
        self.endpoint = 'ecommerce/stores'
        self.store_id = None
        self.order_id = None
        self.lines = StoreOrderLines(self)


    def create(self, store_id, data):
        """
        Add a new order to a store.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "id": string*,
            "customer": object*
            {
                "'id": string*
            },
            "curency_code": string*,
            "order_total": number*,
            "lines": array*
            [
                {
                    "id": string*,
                    "product_id": string*,
                    "product_variant_id": string*,
                    "quantity": integer*,
                    "price": number*
                }
            ]
        }
        """
        self.store_id = store_id
        if 'id' not in data:
            raise KeyError('The order must have an id')
        if 'customer' not in data:
            raise KeyError('The order must have a customer')
        if 'id' not in data['customer']:
            raise KeyError('The order customer must have an id')
        if 'currency_code' not in data:
            raise KeyError('The order must have a currency_code')
        if not re.match(r"^[A-Z]{3}$", data['currency_code']):
            raise ValueError('The currency_code must be a valid 3-letter ISO 4217 currency code')
        if 'order_total' not in data:
            raise KeyError('The order must have an order_total')
        if 'lines' not in data:
            raise KeyError('The order must have at least one order line')
        for line in data['lines']:
            if 'id' not in line:
                raise KeyError('Each order line must have an id')
            if 'product_id' not in line:
                raise KeyError('Each order line must have a product_id')
            if 'product_variant_id' not in line:
                raise KeyError('Each order line must have a product_variant_id')
            if 'quantity' not in line:
                raise KeyError('Each order line must have a quantity')
            if 'price' not in line:
                raise KeyError('Each order line must have a price')
        response = self._mc_client._post(url=self._build_path(store_id, 'orders'), data=data)
        if response is not None:
            self.order_id = response['id']
        else:
            self.order_id = None
        return response


    def all(self, store_id, get_all=False, **queryparams):
        """
        Get information about a store’s orders.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        queryparams['customer_id'] = string
        """
        self.store_id = store_id
        self.order_id = None
        if get_all:
            return self._iterate(url=self._build_path(store_id, 'orders'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(store_id, 'orders'), **queryparams)


    def get(self, store_id, order_id, **queryparams):
        """
        Get information about a specific order.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param order_id: The id for the order in a store.
        :type order_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.store_id = store_id
        self.order_id = order_id
        return self._mc_client._get(url=self._build_path(store_id, 'orders', order_id), **queryparams)


    def update(self, store_id, order_id, data):
        """
        Update a specific order.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param order_id: The id for the order in a store.
        :type order_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        """
        self.store_id = store_id
        self.order_id = order_id
        return self._mc_client._patch(url=self._build_path(store_id, 'orders', order_id), data=data)


    def delete(self, store_id, order_id):
        """
        Delete an order.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param order_id: The id for the order in a store.
        :type order_id: :py:class:`str`
        """
        self.store_id = store_id
        self.order_id = order_id
        return self._mc_client._delete(url=self._build_path(store_id, 'orders', order_id))



# coding=utf-8
"""
The Automation Removed Subscribers endpoint

Note: This is a paid feature

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/automations/removed-subscribers/
Schema: https://api.mailchimp.com/schema/3.0/Automations/RemovedSubscribers/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.helpers import check_email

class AutomationRemovedSubscribers(BaseApi):
    """
    Remove subscribers from an Automation workflow.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(AutomationRemovedSubscribers, self).__init__(*args, **kwargs)
        self.endpoint = 'automations'
        self.workflow_id = None


    # Paid feature
    def create(self, workflow_id, data):
        """
        Remove a subscriber from a specific Automation workflow. You can
        remove a subscriber at any point in an Automation workflow, regardless
        of how many emails they’ve been sent from that workflow. Once they’re
        removed, they can never be added back to the same workflow.

        :param workflow_id: The unique id for the Automation workflow.
        :type workflow_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "email_address": string*
        }
        """
        self.workflow_id = workflow_id
        if 'email_address' not in data:
            raise KeyError('The automation removed subscriber must have an email_address')
        check_email(data['email_address'])
        return self._mc_client._post(url=self._build_path(workflow_id, 'removed-subscribers'), data=data)


    # Paid feature
    def all(self, workflow_id):
        """
        Get information about subscribers who were removed from an Automation
        workflow.

        :param workflow_id: The unique id for the Automation workflow.
        :type workflow_id: :py:class:`str`
        """
        self.workflow_id = workflow_id
        return self._mc_client._get(url=self._build_path(workflow_id, 'removed-subscribers'))




# coding=utf-8
"""
The Click Detail Report Members API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/reports/click-details/members/
Schema: https://api.mailchimp.com/schema/3.0/Reports/ClickDetails/Members/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.helpers import check_subscriber_hash


class ReportClickDetailMembers(BaseApi):
    """
    Get information about specific subscribers who clicked on links in a
    campaign.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ReportClickDetailMembers, self).__init__(*args, **kwargs)
        self.endpoint = 'reports'
        self.campaign_id = None
        self.link_id = None
        self.subscriber_hash = None


    def all(self, campaign_id, link_id, get_all=False, **queryparams):
        """
        Get information about list members who clicked on a specific link in a
        campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param link_id: The id for the link.
        :type link_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.campaign_id = campaign_id
        self.link_id = link_id
        self.subscriber_hash = None
        if get_all:
            return self._iterate(url=self._build_path(campaign_id, 'click-details', link_id, 'members'), **queryparams)
        else:
            return self._mc_client._get(
                url=self._build_path(campaign_id, 'click-details', link_id, 'members'),
                **queryparams
            )


    def get(self, campaign_id, link_id, subscriber_hash, **queryparams):
        """
        Get information about a specific subscriber who clicked a link in a
        specific campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param link_id: The id for the link.
        :type link_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.campaign_id = campaign_id
        self.link_id = link_id
        self.subscriber_hash = subscriber_hash
        return self._mc_client._get(
            url=self._build_path(campaign_id, 'click-details', link_id, 'members', subscriber_hash),
            **queryparams
        )



# coding=utf-8
"""
The E-commerce Store Orders endpoint API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/ecommerce/stores/orders/
Schema: https://api.mailchimp.com/schema/3.0/Ecommerce/Stores/Customers/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class StoreOrderLines(BaseApi):
    """
    Each Order contains one or more Order Lines, which represent a specific
    Product Variant that a Customer purchases.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(StoreOrderLines, self).__init__(*args, **kwargs)
        self.endpoint = 'ecommerce/stores'
        self.store_id = None
        self.order_id = None
        self.line_id = None


    def create(self, store_id, order_id, data):
        """
        Add a new line item to an existing order.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param order_id: The id for the order in a store.
        :type order_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "id": string*,
            "product_id": string*,
            "product_variant_id": string*,
            "quantity": integer*,
            "price": number*
        }
        """
        self.store_id = store_id
        self.order_id = order_id
        if 'id' not in data:
            raise KeyError('The order line must have an id')
        if 'product_id' not in data:
            raise KeyError('The order line must have a product_id')
        if 'product_variant_id' not in data:
            raise KeyError('The order line must have a product_variant_id')
        if 'quantity' not in data:
            raise KeyError('The order line must have a quantity')
        if 'price' not in data:
            raise KeyError('The order line must have a price')
        response = self._mc_client._post(url=self._build_path(store_id, 'orders', order_id, 'lines'))
        if response is not None:
            self.line_id = response['id']
        else:
            self.line_id = None
        return response


    def all(self, store_id, order_id, get_all=False, **queryparams):
        """
        Get information about an order’s line items.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param order_id: The id for the order in a store.
        :type order_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.store_id = store_id
        self.order_id = order_id
        self.line_id = None
        if get_all:
            return self._iterate(url=self._build_path(store_id, 'orders', order_id, 'lines'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(store_id, 'orders', order_id, 'lines'), **queryparams)


    def get(self, store_id, order_id, line_id, **queryparams):
        """
        Get information about a specific order line item.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param order_id: The id for the order in a store.
        :type order_id: :py:class:`str`
        :param line_id: The id for the line item of a cart.
        :type line_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.store_id = store_id
        self.order_id = order_id
        self.line_id = line_id
        return self._mc_client._get(url=self._build_path(store_id, 'orders', order_id, 'lines', line_id), **queryparams)


    def update(self, store_id, order_id, line_id, data):
        """
        Update a specific order line item.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param order_id: The id for the order in a store.
        :type order_id: :py:class:`str`
        :param line_id: The id for the line item of a cart.
        :type line_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        """
        self.store_id = store_id
        self.order_id = order_id
        self.line_id = line_id
        return self._mc_client._patch(url=self._build_path(store_id, 'orders', order_id, 'lines', line_id), data=data)


    def delete(self, store_id, order_id, line_id):
        """
        Delete a specific order line item.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param order_id: The id for the order in a store.
        :type order_id: :py:class:`str`
        :param line_id: The id for the line item of a cart.
        :type line_id: :py:class:`str`
        """
        self.store_id = store_id
        self.order_id = order_id
        self.line_id = line_id
        return self._mc_client._delete(url=self._build_path(store_id, 'orders', order_id, 'lines', line_id))



# coding=utf-8
"""
The Reports API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/reports/
Schema: https://api.mailchimp.com/schema/3.0/Reports/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.reportcampaignabusereports import ReportCampaignAbuseReports
from mailchimp3.entities.reportcampaignadvice import ReportCampaignAdvice
from mailchimp3.entities.reportclickdetailreports import ReportClickDetailReports
from mailchimp3.entities.reportdomainperformance import ReportDomainPerformance
from mailchimp3.entities.reporteepurl import ReportEepURL
from mailchimp3.entities.reportemailactivity import ReportEmailActivity
from mailchimp3.entities.reportlocations import ReportLocations
from mailchimp3.entities.reportsentto import ReportSentTo
from mailchimp3.entities.reportsubreports import ReportSubReports
from mailchimp3.entities.reportunsubscribes import ReportUnsubscribes


class Reports(BaseApi):
    """
    Manage campaign reports for your MailChimp account. All Reports endpoints
    are read-only. MailChimp’s campaign and Automation reports analyze clicks,
    opens, subscribers’ social activity, e-commerce data, and more.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(Reports, self).__init__(*args, **kwargs)
        self.endpoint = 'reports'
        self.campaign_id = None
        self.abuse_reports = ReportCampaignAbuseReports(self)
        self.advice = ReportCampaignAdvice(self)
        self.click_details = ReportClickDetailReports(self)
        self.domain_performance = ReportDomainPerformance(self)
        self.eepurl = ReportEepURL(self)
        self.email_activity = ReportEmailActivity(self)
        self.locations = ReportLocations(self)
        self.sent_to = ReportSentTo(self)
        self.subreports = ReportSubReports(self)
        self.unsubscribes = ReportUnsubscribes(self)


    def all(self, get_all=False, **queryparams):
        """
        Get campaign reports.

        .. note::
            The before_send_time and since_send_time queryparams expect times
            to be listed in the ISO 8601 format in UTC (ex.
            2015-10-21T15:41:36+00:00).

        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        queryparams['type'] = []
        queryparams['before_send_time'] = string
        queryparams['since_send_time'] = string
        """
        self.campaign_id = None
        if get_all:
            return self._iterate(url=self._build_path(), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(), **queryparams)


    def get(self, campaign_id, **queryparams):
        """
        Get report details for a specific sent campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.campaign_id = campaign_id
        return self._mc_client._get(url=self._build_path(campaign_id), **queryparams)



# coding=utf-8
"""
The Lists API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.listabusereports import ListAbuseReports
from mailchimp3.entities.listactivity import ListActivity
from mailchimp3.entities.listclients import ListClients
from mailchimp3.entities.listgrowthhistory import ListGrowthHistory
from mailchimp3.entities.listinterestcategories import ListInterestCategories
from mailchimp3.entities.listmembers import ListMembers
from mailchimp3.entities.listmergefields import ListMergeFields
from mailchimp3.entities.listsegments import ListSegments
from mailchimp3.entities.listsignupforms import ListSignupForms
from mailchimp3.entities.listwebhooks import ListWebhooks
from mailchimp3.helpers import check_email


class Lists(BaseApi):
    """
    A MailChimp list is a powerful and flexible tool that helps you manage your contacts.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(Lists, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None
        self.abuse_reports = ListAbuseReports(self)
        self.activity = ListActivity(self)
        self.clients = ListClients(self)
        self.growth_history = ListGrowthHistory(self)
        self.interest_categories = ListInterestCategories(self)
        self.members = ListMembers(self)
        self.merge_fields = ListMergeFields(self)
        self.segments = ListSegments(self)
        self.signup_forms = ListSignupForms(self)
        self.webhooks = ListWebhooks(self)


    def create(self, data):
        """
        Create a new list in your MailChimp account.

        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "name": string*,
            "contact": object*
            {
                "company": string*,
                "address1": string*,
                "city": string*,
                "state": string*,
                "zip": string*,
                "country": string*
            },
            "permission_reminder": string*,
            "campaign_defaults": object*
            {
                "from_name": string*,
                "from_email": string*,
                "subject": string*,
                "language": string*
            },
            "email_type_option": boolean
        }
        """
        if 'name' not in data:
            raise KeyError('The list must have a name')
        if 'contact' not in data:
            raise KeyError('The list must have a contact')
        if 'company' not in data['contact']:
            raise KeyError('The list contact must have a company')
        if 'address1' not in data['contact']:
            raise KeyError('The list contact must have a address1')
        if 'city' not in data['contact']:
            raise KeyError('The list contact must have a city')
        if 'state' not in data['contact']:
            raise KeyError('The list contact must have a state')
        if 'zip' not in data['contact']:
            raise KeyError('The list contact must have a zip')
        if 'country' not in data['contact']:
            raise KeyError('The list contact must have a country')
        if 'permission_reminder' not in data:
            raise KeyError('The list must have a permission_reminder')
        if 'campaign_defaults' not in data:
            raise KeyError('The list must have a campaign_defaults')
        if 'from_name' not in data['campaign_defaults']:
            raise KeyError('The list campaign_defaults must have a from_name')
        if 'from_email' not in data['campaign_defaults']:
            raise KeyError('The list campaign_defaults must have a from_email')
        check_email(data['campaign_defaults']['from_email'])
        if 'subject' not in data['campaign_defaults']:
            raise KeyError('The list campaign_defaults must have a subject')
        if 'language' not in data['campaign_defaults']:
            raise KeyError('The list campaign_defaults must have a language')
        if 'email_type_option' not in data:
            raise KeyError('The list must have an email_type_option')
        if data['email_type_option'] not in [True, False]:
            raise TypeError('The list email_type_option must be True or False')
        response = self._mc_client._post(url=self._build_path(), data=data)
        if response is not None:
            self.list_id = response['id']
        else:
            self.list_id = None
        return response


    def update_members(self, list_id, data):
        """
        Batch subscribe or unsubscribe list members.

        Only the members array is required in the request body parameters.
        Within the members array, each member requires an email_address
        and either a status or status_if_new. The update_existing parameter
        will also be considered required to help prevent accidental updates
        to existing members and will default to false if not present.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "members": array*
            [
                {
                    "email_address": string*,
                    "status": string* (Must be one of 'subscribed', 'unsubscribed', 'cleaned', or 'pending'),
                    "status_if_new": string* (Must be one of 'subscribed', 'unsubscribed', 'cleaned', or 'pending')
                }
            ],
            "update_existing": boolean*
        }
        """
        self.list_id = list_id
        if 'members' not in data:
            raise KeyError('The update must have at least one member')
        else:
            if not len(data['members']) <= 500:
                raise ValueError('You may only batch sub/unsub 500 members at a time')
        for member in data['members']:
            if 'email_address' not in member:
                raise KeyError('Each list member must have an email_address')
            check_email(member['email_address'])
            if 'status' not in member and 'status_if_new' not in member:
                raise KeyError('Each list member must have either a status or a status_if_new')
            valid_statuses = ['subscribed', 'unsubscribed', 'cleaned', 'pending']
            if 'status' in member and member['status'] not in valid_statuses:
                raise ValueError('The list member status must be one of "subscribed", "unsubscribed", "cleaned", or '
                                 '"pending"')
            if 'status_if_new' in member and member['status_if_new'] not in valid_statuses:
                raise ValueError('The list member status_if_new must be one of "subscribed", "unsubscribed", '
                                 '"cleaned", or "pending"')
        if 'update_existing' not in data:
            data['update_existing'] = False
        return self._mc_client._post(url=self._build_path(list_id), data=data)


    def all(self, get_all=False, **queryparams):
        """
        Get information about all lists in the account.

        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        queryparams['before_date_created'] = string
        queryparams['since_date_created'] = string
        queryparams['before_campaign_last_sent'] = string
        queryparams['since_campaign_last_sent'] = string
        queryparams['email'] = string
        queryparams['sort_field'] = string (Must be 'date_created')
        queryparams['sort_dir'] = string (Must be one of 'ASC' or 'DESC')
        """
        self.list_id = None
        if get_all:
            return self._iterate(url=self._build_path(), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(), **queryparams)


    def get(self, list_id, **queryparams):
        """
        Get information about a specific list in your MailChimp account.
        Results include list members who have signed up but haven’t confirmed
        their subscription yet and unsubscribed or cleaned.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.list_id = list_id
        return self._mc_client._get(url=self._build_path(list_id), **queryparams)


    def update(self, list_id, data):
        """
        Update the settings for a specific list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "name": string*,
            "contact": object*
            {
                "company": string*,
                "address1": string*,
                "city": string*,
                "state": string*,
                "zip": string*,
                "country": string*
            },
            "permission_reminder": string*,
            "campaign_defaults": object*
            {
                "from_name": string*,
                "from_email": string*,
                "subject": string*,
                "language": string*
            },
            "email_type_option": boolean
        }
        """
        self.list_id = list_id
        if 'name' not in data:
            raise KeyError('The list must have a name')
        if 'contact' not in data:
            raise KeyError('The list must have a contact')
        if 'company' not in data['contact']:
            raise KeyError('The list contact must have a company')
        if 'address1' not in data['contact']:
            raise KeyError('The list contact must have a address1')
        if 'city' not in data['contact']:
            raise KeyError('The list contact must have a city')
        if 'state' not in data['contact']:
            raise KeyError('The list contact must have a state')
        if 'zip' not in data['contact']:
            raise KeyError('The list contact must have a zip')
        if 'country' not in data['contact']:
            raise KeyError('The list contact must have a country')
        if 'permission_reminder' not in data:
            raise KeyError('The list must have a permission_reminder')
        if 'campaign_defaults' not in data:
            raise KeyError('The list must have a campaign_defaults')
        if 'from_name' not in data['campaign_defaults']:
            raise KeyError('The list campaign_defaults must have a from_name')
        if 'from_email' not in data['campaign_defaults']:
            raise KeyError('The list campaign_defaults must have a from_email')
        check_email(data['campaign_defaults']['from_email'])
        if 'subject' not in data['campaign_defaults']:
            raise KeyError('The list campaign_defaults must have a subject')
        if 'language' not in data['campaign_defaults']:
            raise KeyError('The list campaign_defaults must have a language')
        if 'email_type_option' not in data:
            raise KeyError('The list must have an email_type_option')
        if data['email_type_option'] not in [True, False]:
            raise TypeError('The list email_type_option must be True or False')
        return self._mc_client._patch(url=self._build_path(list_id), data=data)


    def delete(self, list_id):
        """
        Delete a list from your MailChimp account. If you delete a list,
        you’ll lose the list history—including subscriber activity,
        unsubscribes, complaints, and bounces. You’ll also lose subscribers’
        email addresses, unless you exported and backed up your list.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        """
        self.list_id = list_id
        return self._mc_client._delete(url=self._build_path(list_id))




# coding=utf-8
"""
The Automation Email Queue endpoint

Note: This is a paid feature

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/automations/emails/queue/
Schema: https://api.mailchimp.com/schema/3.0/Automations/Emails/Queue/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.helpers import check_email, check_subscriber_hash


class AutomationEmailQueues(BaseApi):
    """
    Manage list member queues for Automation emails.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(AutomationEmailQueues, self).__init__(*args, **kwargs)
        self.endpoint = 'automations'
        self.workflow_id = None
        self.email_id = None
        self.subscriber_hash = None

    # Paid feature
    def create(self, workflow_id, email_id, data):
        """
        Manually add a subscriber to a workflow, bypassing the default trigger
        settings. You can also use this endpoint to trigger a series of
        automated emails in an API 3.0 workflow type or add subscribers to an
        automated email queue that uses the API request delay type.

        :param workflow_id: The unique id for the Automation workflow.
        :type workflow_id: :py:class:`str`
        :param email_id: The unique id for the Automation workflow email.
        :type email_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "email_address": string*
        }
        """
        self.workflow_id = workflow_id
        self.email_id = email_id
        if 'email_address' not in data:
            raise KeyError('The automation email queue must have an email_address')
        check_email(data['email_address'])
        response = self._mc_client._post(
            url=self._build_path(workflow_id, 'emails', email_id, 'queue'),
            data=data
        )
        if response is not None:
            self.subscriber_hash = response['id']
        else:
            self.subscriber_hash = None
        return response


    # Paid feature
    def all(self, workflow_id, email_id):
        """
        Get information about an Automation email queue.

        :param workflow_id: The unique id for the Automation workflow.
        :type workflow_id: :py:class:`str`
        :param email_id: The unique id for the Automation workflow email.
        :type email_id: :py:class:`str`
        """
        self.workflow_id = workflow_id
        self.email_id = email_id
        self.subscriber_hash = None
        return self._mc_client._get(url=self._build_path(workflow_id, 'emails', email_id, 'queue'))


    # Paid feature
    def get(self, workflow_id, email_id, subscriber_hash):
        """
        Get information about a specific subscriber in an Automation email
        queue.

        :param workflow_id: The unique id for the Automation workflow.
        :type workflow_id: :py:class:`str`
        :param email_id: The unique id for the Automation workflow email.
        :type email_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.workflow_id = workflow_id
        self.email_id = email_id
        self.subscriber_hash = subscriber_hash
        return self._mc_client._get(url=self._build_path(workflow_id, 'emails', email_id, 'queue', subscriber_hash))




# coding=utf-8
"""
The Automations API endpoint

Note: This is a paid feature

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/automations/
Schema: https://api.mailchimp.com/schema/3.0/Automations/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.automationactions import AutomationActions
from mailchimp3.entities.automationemails import AutomationEmails
from mailchimp3.entities.automationremovedsubscribers import AutomationRemovedSubscribers


class Automations(BaseApi):
    """
    Automation is a paid feature that lets you build a series of triggered
    emails that are sent to subscribers over a set period of time. Use the
    Automation API calls to manage Automation workflows, emails, and queues.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(Automations, self).__init__(*args, **kwargs)
        self.endpoint = 'automations'
        self.workflow_id = None
        self.actions = AutomationActions(self)
        self.emails = AutomationEmails(self)
        self.removed_subscribers = AutomationRemovedSubscribers(self)


    # Paid feature
    def all(self, get_all=False, **queryparams):
        """
        Get a summary of an account’s Automations.

        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: the query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.workflow_id = None
        if get_all:
            return self._iterate(url=self._build_path(), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(), **queryparams)


    # Paid feature
    def get(self, workflow_id, **queryparams):
        """
        Get a summary of an individual Automation workflow’s settings and
        content. The trigger_settings object returns information for the first
        email in the workflow.

        :param workflow_id: The unique id for the Automation workflow
        :type workflow_id: :py:class:`str`
        :param queryparams: the query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.workflow_id = workflow_id
        return self._mc_client._get(url=self._build_path(workflow_id), **queryparams)



# coding=utf-8
"""
The Batch Operations API Endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/batches/
Schema: https://api.mailchimp.com/schema/3.0/Batches/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class BatchOperations(BaseApi):
    """
    Use batch operations to complete multiple operations with a single call.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(BatchOperations, self).__init__(*args, **kwargs)
        self.endpoint = 'batches'
        self.batch_id = None
        self.operation_status = None


    def create(self, data):
        """
        Begin processing a batch operations request.

        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "operations": array*
            [
                {
                    "method": string* (Must be one of "GET", "POST", "PUT", "PATCH", or "DELETE")
                    "path": string*,
                }
            ]
        }
        """
        if 'operations' not in data:
            raise KeyError('The batch must have operations')
        for op in data['operations']:
            if 'method' not in op:
                raise KeyError('The batch operation must have a method')
            if op['method'] not in ['GET', 'POST', 'PUT', 'PATCH', 'DELETE']:
                raise ValueError('The batch operation method must be one of "GET", "POST", "PUT", "PATCH", '
                                 'or "DELETE", not {0}'.format(op['method']))
            if 'path' not in op:
                raise KeyError('The batch operation must have a path')
        return self._mc_client._post(url=self._build_path(), data=data)


    def all(self, get_all=False, **queryparams):
        """
        Get a summary of batch requests that have been made.

        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.batch_id = None
        self.operation_status = None
        if get_all:
            return self._iterate(url=self._build_path(), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(), **queryparams)


    def get(self, batch_id, **queryparams):
        """
        Get the status of a batch request.

        :param batch_id: The unique id for the batch operation.
        :type batch_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.batch_id = batch_id
        self.operation_status = None
        return self._mc_client._get(url=self._build_path(batch_id), **queryparams)


    def delete(self, batch_id):
        """
        Stops a batch request from running. Since only one batch request is
        run at a time, this can be used to cancel a long running request. The
        results of any completed operations will not be available after this
        call.

        :param batch_id: The unique id for the batch operation.
        :type batch_id: :py:class:`str`
        """
        self.batch_id = batch_id
        self.operation_status = None
        return self._mc_client._delete(url=self._build_path(batch_id))



# coding=utf-8
"""
The Automations API endpoint actions

Note: This is a paid feature

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/automations/
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class AutomationActions(BaseApi):
    """
    Actions for the Automations endpoint.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(AutomationActions, self).__init__(*args, **kwargs)
        self.endpoint = 'automations'
        self.workflow_id = None


    # Paid feature
    def pause(self, workflow_id):
        """
        Pause all emails in a specific Automation workflow.

        :param workflow_id: The unique id for the Automation workflow.
        :type workflow_id: :py:class:`str`
        """
        self.workflow_id = workflow_id
        return self._mc_client._post(url=self._build_path(workflow_id, 'actions/pause-all-emails'))


    # Paid feature
    def start(self, workflow_id):
        """
        Start all emails in an Automation workflow.

        :param workflow_id: The unique id for the Automation workflow.
        :type workflow_id: :py:class:`str`
        """
        self.workflow_id = workflow_id
        return self._mc_client._post(url=self._build_path(workflow_id, 'actions/start-all-emails'))
